@@ -18,15 +18,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#define COBJMACROS
-
-#include <stdarg.h>
-
-#include "windef.h"
-#include "winbase.h"
-#include "ole2.h"
-#include "rpcproxy.h"
-#include "comsvcs.h"
+#include "comsvcs_private.h"
 #include "wine/heap.h"
 #include "wine/debug.h"
 #include "initguid.h"
@@ -34,8 +26,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(comsvcs);
 
-static HINSTANCE COMSVCS_hInstance;
-
 typedef struct dispensermanager
 {
     IDispenserManager IDispenserManager_iface;
@@ -356,18 +346,6 @@ static HRESULT WINAPI dispenser_manager_cf_CreateInstance(IClassFactory *iface,
     return ret;
 }
 
-BOOL WINAPI DllMain(HINSTANCE hinst, DWORD reason, LPVOID lpv)
-{
-    switch(reason)
-    {
-    case DLL_PROCESS_ATTACH:
-        COMSVCS_hInstance = hinst;
-        DisableThreadLibraryCalls(hinst);
-        break;
-    }
-    return TRUE;
-}
-
 static HRESULT WINAPI comsvcscf_QueryInterface(IClassFactory *iface, REFIID riid, void **ppv )
 {
     *ppv = NULL;
@@ -1009,8 +987,18 @@ static const IClassFactoryVtbl newmoniker_cf_vtbl =
     comsvcscf_LockServer
 };
 
+static const IClassFactoryVtbl group_manager_cf_vtbl =
+{
+    comsvcscf_QueryInterface,
+    comsvcscf_AddRef,
+    comsvcscf_Release,
+    group_manager_create,
+    comsvcscf_LockServer
+};
+
 static IClassFactory DispenserManageFactory = { &comsvcscf_vtbl };
 static IClassFactory NewMonikerFactory = { &newmoniker_cf_vtbl };
+static IClassFactory GroupManagerFactory = { &group_manager_cf_vtbl };
 
 /******************************************************************
  * DllGetClassObject
@@ -1027,31 +1015,12 @@ HRESULT WINAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, void **ppv)
         TRACE("(CLSID_NewMoniker %s %p)\n", debugstr_guid(riid), ppv);
         return IClassFactory_QueryInterface(&NewMonikerFactory, riid, ppv);
     }
+    else if (IsEqualGUID(&CLSID_SharedPropertyGroupManager, rclsid))
+    {
+        TRACE("(CLSID_SharedPropertyGroupManager %s %p)\n", debugstr_guid(riid), ppv);
+        return IClassFactory_QueryInterface(&GroupManagerFactory, riid, ppv);
+    }
 
     FIXME("%s %s %p\n", debugstr_guid(rclsid), debugstr_guid(riid), ppv);
     return CLASS_E_CLASSNOTAVAILABLE;
 }
-
-/******************************************************************
- * DllCanUnloadNow
- */
-HRESULT WINAPI DllCanUnloadNow(void)
-{
-    return S_FALSE;
-}
-
-/***********************************************************************
- *		DllRegisterServer (comsvcs.@)
- */
-HRESULT WINAPI DllRegisterServer(void)
-{
-    return __wine_register_resources( COMSVCS_hInstance );
-}
-
-/***********************************************************************
- *		DllUnregisterServer (comsvcs.@)
- */
-HRESULT WINAPI DllUnregisterServer(void)
-{
-    return __wine_unregister_resources( COMSVCS_hInstance );
-}
