@@ -82,6 +82,11 @@ static inline const char *debugstr_diobjectdataformat( const DIOBJECTDATAFORMAT
                              debugstr_guid( data->pguid ), data->dwOfs, data->dwType, data->dwFlags );
 }
 
+static inline BOOL is_exclusively_acquired( struct dinput_device *device )
+{
+    return device->status == STATUS_ACQUIRED && (device->dwCoopLevel & DISCL_EXCLUSIVE);
+}
+
 /******************************************************************************
  *	Various debugging tools
  */
@@ -240,11 +245,13 @@ LPDIOBJECTDATAFORMAT dataformat_to_odf_by_type(LPCDIDATAFORMAT df, int n, DWORD
 }
 
 static BOOL match_device_object( DIDATAFORMAT *device_format, DIDATAFORMAT *user_format,
-                                 const DIDATAFORMAT *format, const DIOBJECTDATAFORMAT *match_obj )
+                                 const DIDATAFORMAT *format, const DIOBJECTDATAFORMAT *match_obj, DWORD version )
 {
     DWORD i, device_instance, instance = DIDFT_GETINSTANCE( match_obj->dwType );
     DIOBJECTDATAFORMAT *device_obj, *user_obj;
 
+    if (version < 0x0700 && instance == 0xff) instance = 0xffff;
+
     for (i = 0; i < device_format->dwNumObjs; i++)
     {
         user_obj = user_format->rgodf + i;
@@ -292,7 +299,7 @@ static HRESULT dinput_device_init_user_format( struct dinput_device *impl, const
     {
         match_obj = format->rgodf + i;
 
-        if (!match_device_object( device_format, user_format, format, match_obj ))
+        if (!match_device_object( device_format, user_format, format, match_obj, impl->dinput->dwVersion ))
         {
             WARN( "object %s not found\n", debugstr_diobjectdataformat( match_obj ) );
             if (!(match_obj->dwType & DIDFT_OPTIONAL)) goto failed;
@@ -575,7 +582,7 @@ static HRESULT WINAPI dinput_device_Acquire( IDirectInputDevice8W *iface )
     TRACE( "iface %p.\n", iface );
 
     EnterCriticalSection( &impl->crit );
-    if (impl->acquired)
+    if (impl->status == STATUS_ACQUIRED)
         hr = DI_NOEFFECT;
     else if (!impl->user_format)
         hr = DIERR_INVALIDPARAM;
@@ -583,8 +590,8 @@ static HRESULT WINAPI dinput_device_Acquire( IDirectInputDevice8W *iface )
         hr = DIERR_OTHERAPPHASPRIO;
     else
     {
-        impl->acquired = TRUE;
-        if (FAILED(hr = impl->vtbl->acquire( iface ))) impl->acquired = FALSE;
+        impl->status = STATUS_ACQUIRED;
+        if (FAILED(hr = impl->vtbl->acquire( iface ))) impl->status = STATUS_UNACQUIRED;
     }
     LeaveCriticalSection( &impl->crit );
     if (hr != DI_OK) return hr;
@@ -607,9 +614,9 @@ static HRESULT WINAPI dinput_device_Unacquire( IDirectInputDevice8W *iface )
     TRACE( "iface %p.\n", iface );
 
     EnterCriticalSection( &impl->crit );
-    if (!impl->acquired) hr = DI_NOEFFECT;
+    if (impl->status != STATUS_ACQUIRED) hr = DI_NOEFFECT;
     else hr = impl->vtbl->unacquire( iface );
-    impl->acquired = FALSE;
+    impl->status = STATUS_UNACQUIRED;
     LeaveCriticalSection( &impl->crit );
     if (hr != DI_OK) return hr;
 
@@ -640,7 +647,7 @@ static HRESULT WINAPI dinput_device_SetDataFormat( IDirectInputDevice8W *iface,
 
     if (format->dwSize != sizeof(DIDATAFORMAT)) return DIERR_INVALIDPARAM;
     if (format->dwObjSize != sizeof(DIOBJECTDATAFORMAT)) return DIERR_INVALIDPARAM;
-    if (This->acquired) return DIERR_ACQUIRED;
+    if (This->status == STATUS_ACQUIRED) return DIERR_ACQUIRED;
 
     EnterCriticalSection(&This->crit);
 
@@ -689,7 +696,7 @@ static HRESULT WINAPI dinput_device_SetCooperativeLevel( IDirectInputDevice8W *i
 
     /* Store the window which asks for the mouse */
     EnterCriticalSection(&This->crit);
-    if (This->acquired) hr = DIERR_ACQUIRED;
+    if (This->status == STATUS_ACQUIRED) hr = DIERR_ACQUIRED;
     else
     {
         This->win = hwnd;
@@ -907,6 +914,19 @@ static HRESULT enum_object_filter_init( struct dinput_device *impl, DIPROPHEADER
 
 static HRESULT check_property( struct dinput_device *impl, const GUID *guid, const DIPROPHEADER *header, BOOL set )
 {
+    switch (LOWORD( guid ))
+    {
+    case (DWORD_PTR)DIPROP_VIDPID:
+    case (DWORD_PTR)DIPROP_TYPENAME:
+    case (DWORD_PTR)DIPROP_USERNAME:
+    case (DWORD_PTR)DIPROP_KEYNAME:
+    case (DWORD_PTR)DIPROP_LOGICALRANGE:
+    case (DWORD_PTR)DIPROP_PHYSICALRANGE:
+    case (DWORD_PTR)DIPROP_APPDATA:
+        if (impl->dinput->dwVersion < 0x0800) return DIERR_UNSUPPORTED;
+        break;
+    }
+
     switch (LOWORD( guid ))
     {
     case (DWORD_PTR)DIPROP_INSTANCENAME:
@@ -994,7 +1014,7 @@ static HRESULT check_property( struct dinput_device *impl, const GUID *guid, con
         case (DWORD_PTR)DIPROP_BUFFERSIZE:
         case (DWORD_PTR)DIPROP_PHYSICALRANGE:
         case (DWORD_PTR)DIPROP_LOGICALRANGE:
-            if (impl->acquired) return DIERR_ACQUIRED;
+            if (impl->status == STATUS_ACQUIRED) return DIERR_ACQUIRED;
             break;
         case (DWORD_PTR)DIPROP_FFLOAD:
         case (DWORD_PTR)DIPROP_GRANULARITY:
@@ -1056,6 +1076,10 @@ static HRESULT check_property( struct dinput_device *impl, const GUID *guid, con
             if (!impl->object_properties) return DIERR_UNSUPPORTED;
             break;
 
+        case (DWORD_PTR)DIPROP_FFLOAD:
+            if (!(impl->caps.dwFlags & DIDC_FORCEFEEDBACK)) return DIERR_UNSUPPORTED;
+            if (!is_exclusively_acquired( impl )) return DIERR_NOTEXCLUSIVEACQUIRED;
+            /* fallthrough */
         case (DWORD_PTR)DIPROP_PRODUCTNAME:
         case (DWORD_PTR)DIPROP_INSTANCENAME:
         case (DWORD_PTR)DIPROP_VIDPID:
@@ -1173,6 +1197,7 @@ static HRESULT dinput_device_get_property( IDirectInputDevice8W *iface, const GU
     case (DWORD_PTR)DIPROP_VIDPID:
     case (DWORD_PTR)DIPROP_JOYSTICKID:
     case (DWORD_PTR)DIPROP_GUIDANDPATH:
+    case (DWORD_PTR)DIPROP_FFLOAD:
         return impl->vtbl->get_property( iface, LOWORD( guid ), header, NULL );
 
     case (DWORD_PTR)DIPROP_RANGE:
@@ -1379,7 +1404,7 @@ static HRESULT WINAPI dinput_device_set_property( IDirectInputDevice8W *iface, c
         const DIPROPDWORD *value = (const DIPROPDWORD *)header;
         if (!impl->vtbl->send_device_gain) return DIERR_UNSUPPORTED;
         impl->device_gain = value->dwData;
-        if (!impl->acquired || !(impl->dwCoopLevel & DISCL_EXCLUSIVE)) return DI_OK;
+        if (!is_exclusively_acquired( impl )) return DI_OK;
         return impl->vtbl->send_device_gain( iface, impl->device_gain );
     }
     case (DWORD_PTR)DIPROP_AXISMODE:
@@ -1528,7 +1553,9 @@ static HRESULT WINAPI dinput_device_GetDeviceState( IDirectInputDevice8W *iface,
     IDirectInputDevice2_Poll( iface );
 
     EnterCriticalSection( &impl->crit );
-    if (!impl->acquired)
+    if (impl->status == STATUS_UNPLUGGED)
+        hr = DIERR_INPUTLOST;
+    else if (impl->status != STATUS_ACQUIRED)
         hr = DIERR_NOTACQUIRED;
     else if (!(user_format = impl->user_format))
         hr = DIERR_INVALIDPARAM;
@@ -1580,7 +1607,8 @@ static HRESULT WINAPI dinput_device_GetDeviceData( IDirectInputDevice8W *iface,
     if (This->dinput->dwVersion == 0x0800 || size == sizeof(DIDEVICEOBJECTDATA_DX3))
     {
         if (!This->queue_len) return DIERR_NOTBUFFERED;
-        if (!This->acquired) return DIERR_NOTACQUIRED;
+        if (This->status == STATUS_UNPLUGGED) return DIERR_INPUTLOST;
+        if (This->status != STATUS_ACQUIRED) return DIERR_NOTACQUIRED;
     }
 
     if (!This->queue_len)
@@ -1659,7 +1687,7 @@ static HRESULT WINAPI dinput_device_CreateEffect( IDirectInputDevice8W *iface, c
     if (!params) return DI_OK;
 
     flags = params->dwSize == sizeof(DIEFFECT_DX6) ? DIEP_ALLPARAMS : DIEP_ALLPARAMS_DX5;
-    if (!impl->acquired || !(impl->dwCoopLevel & DISCL_EXCLUSIVE)) flags |= DIEP_NODOWNLOAD;
+    if (!is_exclusively_acquired( impl )) flags |= DIEP_NODOWNLOAD;
     hr = IDirectInputEffect_SetParameters( *out, params, flags );
     if (FAILED(hr)) goto failed;
     return DI_OK;
@@ -1757,9 +1785,22 @@ static HRESULT WINAPI dinput_device_GetEffectInfo( IDirectInputDevice8W *iface,
 
 static HRESULT WINAPI dinput_device_GetForceFeedbackState( IDirectInputDevice8W *iface, DWORD *out )
 {
-    FIXME( "iface %p, out %p stub!\n", iface, out );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8W( iface );
+    HRESULT hr = DI_OK;
+
+    TRACE( "iface %p, out %p.\n", iface, out );
+
     if (!out) return E_POINTER;
-    return DIERR_UNSUPPORTED;
+    *out = 0;
+
+    if (!(impl->caps.dwFlags & DIDC_FORCEFEEDBACK)) return DIERR_UNSUPPORTED;
+
+    EnterCriticalSection( &impl->crit );
+    if (!is_exclusively_acquired( impl )) hr = DIERR_NOTEXCLUSIVEACQUIRED;
+    else *out = impl->force_feedback_state;
+    LeaveCriticalSection( &impl->crit );
+
+    return hr;
 }
 
 static HRESULT WINAPI dinput_device_SendForceFeedbackCommand( IDirectInputDevice8W *iface, DWORD command )
@@ -1784,7 +1825,7 @@ static HRESULT WINAPI dinput_device_SendForceFeedbackCommand( IDirectInputDevice
     if (!impl->vtbl->send_force_feedback_command) return DIERR_UNSUPPORTED;
 
     EnterCriticalSection( &impl->crit );
-    if (!impl->acquired || !(impl->dwCoopLevel & DISCL_EXCLUSIVE)) hr = DIERR_NOTEXCLUSIVEACQUIRED;
+    if (!is_exclusively_acquired( impl )) hr = DIERR_NOTEXCLUSIVEACQUIRED;
     else hr = impl->vtbl->send_force_feedback_command( iface, command, FALSE );
     LeaveCriticalSection( &impl->crit );
 
@@ -1819,7 +1860,8 @@ static HRESULT WINAPI dinput_device_Poll( IDirectInputDevice8W *iface )
     HRESULT hr = DI_NOEFFECT;
 
     EnterCriticalSection( &impl->crit );
-    if (!impl->acquired) hr = DIERR_NOTACQUIRED;
+    if (impl->status == STATUS_UNPLUGGED) hr = DIERR_INPUTLOST;
+    else if (impl->status != STATUS_ACQUIRED) hr = DIERR_NOTACQUIRED;
     LeaveCriticalSection( &impl->crit );
     if (FAILED(hr)) return hr;
 
@@ -1974,7 +2016,7 @@ static HRESULT WINAPI dinput_device_SetActionMap( IDirectInputDevice8W *iface, D
         break;
     }
 
-    if (impl->acquired) return DIERR_ACQUIRED;
+    if (impl->status == STATUS_ACQUIRED) return DIERR_ACQUIRED;
 
     data_format.dwSize = sizeof(data_format);
     data_format.dwObjSize = sizeof(DIOBJECTDATAFORMAT);
@@ -2133,6 +2175,7 @@ HRESULT dinput_device_alloc( SIZE_T size, const struct dinput_device_vtbl *vtbl,
     This->caps.dwFlags = DIDC_ATTACHED | DIDC_EMULATED;
     This->device_format = format;
     This->device_gain = 10000;
+    This->force_feedback_state = DIGFFS_STOPPED | DIGFFS_EMPTY;
     InitializeCriticalSection( &This->crit );
     This->dinput = dinput;
     IDirectInput_AddRef( &dinput->IDirectInput7A_iface );
