@@ -40,7 +40,6 @@ DEFINE_GUID(GUID_NULL, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
 #include "d3d9.h"
 #include "dxva2api.h"
 
-#include "wine/heap.h"
 #include "wine/test.h"
 
 static ULONG get_refcount(void *iface)
@@ -145,7 +144,7 @@ static ULONG WINAPI test_media_stream_Release(IMFMediaStream *iface)
     if (!refcount)
     {
         IMFMediaEventQueue_Release(stream->event_queue);
-        heap_free(stream);
+        free(stream);
     }
 
     return refcount;
@@ -297,7 +296,7 @@ static ULONG WINAPI test_source_Release(IMFMediaSource *iface)
     if (!refcount)
     {
         IMFMediaEventQueue_Release(source->event_queue);
-        heap_free(source);
+        free(source);
     }
 
     return refcount;
@@ -362,6 +361,8 @@ static HRESULT WINAPI test_source_CreatePresentationDescriptor(IMFMediaSource *i
             ok(hr == S_OK, "Failed to set attribute, hr %#x.\n", hr);
             hr = IMFMediaType_SetGUID(media_type, &MF_MT_SUBTYPE, &MFAudioFormat_PCM);
             ok(hr == S_OK, "Failed to set attribute, hr %#x.\n", hr);
+            hr = IMFMediaType_SetUINT32(media_type, &MF_MT_AUDIO_BITS_PER_SAMPLE, 32);
+            ok(hr == S_OK, "Failed to set attribute, hr %#x.\n", hr);
 
             hr = MFCreateStreamDescriptor(i, 1, &media_type, &sds[i]);
             ok(hr == S_OK, "Failed to create stream descriptor, hr %#x.\n", hr);
@@ -486,7 +487,7 @@ static struct test_media_stream *create_test_stream(DWORD stream_index, IMFMedia
     BOOL selected;
     HRESULT hr;
 
-    stream = heap_alloc_zero(sizeof(*stream));
+    stream = calloc(1, sizeof(*stream));
     stream->IMFMediaStream_iface.lpVtbl = &test_media_stream_vtbl;
     stream->refcount = 1;
     hr = MFCreateEventQueue(&stream->event_queue);
@@ -507,7 +508,7 @@ static IMFMediaSource *create_test_source(int stream_count)
     struct test_source *source;
     int i;
 
-    source = heap_alloc_zero(sizeof(*source));
+    source = calloc(1, sizeof(*source));
     source->IMFMediaSource_iface.lpVtbl = &test_source_vtbl;
     source->refcount = 1;
     source->stream_count = stream_count;
@@ -604,9 +605,7 @@ static ULONG WINAPI async_callback_Release(IMFSourceReaderCallback *iface)
     ULONG refcount = InterlockedDecrement(&callback->refcount);
 
     if (!refcount)
-    {
-        heap_free(callback);
-    }
+        free(callback);
 
     return refcount;
 }
@@ -644,7 +643,7 @@ static struct async_callback *create_async_callback(void)
 {
     struct async_callback *callback;
 
-    callback = heap_alloc(sizeof(*callback));
+    callback = calloc(1, sizeof(*callback));
     callback->IMFSourceReaderCallback_iface.lpVtbl = &async_callback_vtbl;
     callback->refcount = 1;
 
@@ -750,9 +749,6 @@ static void test_source_reader(void)
     if (SUCCEEDED(hr))
         IMFMediaType_Release(mediatype);
 
-    hr = IMFSourceReader_GetNativeMediaType(reader, MF_SOURCE_READER_FIRST_AUDIO_STREAM, 1, &mediatype);
-    ok(hr == MF_E_NO_MORE_TYPES, "Unexpected hr %#x.\n", hr);
-
     /* Current media type. */
     hr = IMFSourceReader_GetCurrentMediaType(reader, MF_SOURCE_READER_FIRST_VIDEO_STREAM, &mediatype);
     ok(hr == MF_E_INVALIDSTREAMNUMBER, "Unexpected hr %#x.\n", hr);
@@ -897,6 +893,7 @@ static void test_source_reader_from_media_source(void)
     struct async_callback *callback;
     IMFSourceReader *reader;
     IMFMediaSource *source;
+    IMFMediaType *media_type;
     HRESULT hr;
     DWORD actual_index, stream_flags;
     IMFSample *sample;
@@ -1042,6 +1039,54 @@ static void test_source_reader_from_media_source(void)
     IMFSourceReader_Release(reader);
     IMFMediaSource_Release(source);
 
+    /* Request a non-native bit depth. */
+    source = create_test_source(1);
+    ok(!!source, "Failed to create test source.\n");
+
+    hr = MFCreateSourceReaderFromMediaSource(source, NULL, &reader);
+    ok(hr == S_OK, "Failed to create source reader, hr %#x.\n", hr);
+
+    hr = MFCreateMediaType(&media_type);
+    ok(hr == S_OK, "Failed to create media type, hr %#x.\n", hr);
+    hr = IMFMediaType_SetGUID(media_type, &MF_MT_MAJOR_TYPE, &MFMediaType_Audio);
+    ok(hr == S_OK, "Failed to set attribute, hr %#x.\n", hr);
+    hr = IMFMediaType_SetGUID(media_type, &MF_MT_SUBTYPE, &MFAudioFormat_PCM);
+    ok(hr == S_OK, "Failed to set attribute, hr %#x.\n", hr);
+    hr = IMFMediaType_SetUINT32(media_type, &MF_MT_AUDIO_BITS_PER_SAMPLE, 16);
+    ok(hr == S_OK, "Failed to set attribute, hr %#x.\n", hr);
+
+    hr = IMFSourceReader_SetCurrentMediaType(reader, 0, NULL, media_type);
+    ok(hr == MF_E_TOPO_CODEC_NOT_FOUND, "Unexpected success setting current media type, hr %#x.\n", hr);
+
+    IMFMediaType_Release(media_type);
+
+    hr = MFCreateMediaType(&media_type);
+    ok(hr == S_OK, "Failed to create media type, hr %#x.\n", hr);
+    hr = IMFMediaType_SetGUID(media_type, &MF_MT_MAJOR_TYPE, &MFMediaType_Audio);
+    ok(hr == S_OK, "Failed to set attribute, hr %#x.\n", hr);
+    hr = IMFMediaType_SetGUID(media_type, &MF_MT_SUBTYPE, &MFAudioFormat_PCM);
+    ok(hr == S_OK, "Failed to set attribute, hr %#x.\n", hr);
+    hr = IMFMediaType_SetUINT32(media_type, &MF_MT_AUDIO_BITS_PER_SAMPLE, 32);
+    ok(hr == S_OK, "Failed to set attribute, hr %#x.\n", hr);
+
+    hr = IMFSourceReader_SetCurrentMediaType(reader, 0, NULL, media_type);
+    ok(hr == S_OK, "Failed to set current media type, hr %#x.\n", hr);
+
+    hr = IMFSourceReader_SetStreamSelection(reader, 0, TRUE);
+    ok(hr == S_OK, "Failed to select a stream, hr %#x.\n", hr);
+
+    hr = IMFSourceReader_ReadSample(reader, 0, 0, &actual_index, &stream_flags, &timestamp, &sample);
+    ok(hr == S_OK, "Failed to get a sample, hr %#x.\n", hr);
+    ok(actual_index == 0, "Unexpected stream index %u\n", actual_index);
+    ok(!stream_flags, "Unexpected stream flags %#x.\n", stream_flags);
+    ok(timestamp == 123, "Unexpected timestamp.\n");
+    ok(!!sample, "Expected sample object.\n");
+    IMFSample_Release(sample);
+
+    IMFMediaType_Release(media_type);
+    IMFSourceReader_Release(reader);
+    IMFMediaSource_Release(source);
+
     /* Async mode. */
     source = create_test_source(3);
     ok(!!source, "Failed to create test source.\n");
