@@ -1974,6 +1974,11 @@ struct device_notification_details
 {
     DWORD (CALLBACK *cb)(HANDLE handle, DWORD flags, DEV_BROADCAST_HDR *header);
     HANDLE handle;
+    union
+    {
+        DEV_BROADCAST_HDR header;
+        DEV_BROADCAST_DEVICEINTERFACE_W iface;
+    } filter;
 };
 
 static HANDLE device_notify_thread;
@@ -1985,16 +1990,35 @@ struct device_notify_registration
     struct device_notification_details details;
 };
 
+static BOOL notification_filter_matches( DEV_BROADCAST_HDR *filter, DEV_BROADCAST_HDR *event )
+{
+    if (!filter->dbch_devicetype) return TRUE;
+    if (filter->dbch_devicetype != event->dbch_devicetype) return FALSE;
+
+    if (filter->dbch_devicetype == DBT_DEVTYP_DEVICEINTERFACE)
+    {
+        DEV_BROADCAST_DEVICEINTERFACE_W *filter_iface = (DEV_BROADCAST_DEVICEINTERFACE_W *)filter;
+        DEV_BROADCAST_DEVICEINTERFACE_W *event_iface = (DEV_BROADCAST_DEVICEINTERFACE_W *)event;
+        if (filter_iface->dbcc_size == offsetof(DEV_BROADCAST_DEVICEINTERFACE_W, dbcc_classguid)) return TRUE;
+        return IsEqualGUID( &filter_iface->dbcc_classguid, &event_iface->dbcc_classguid );
+    }
+
+    FIXME( "Filter dbch_devicetype %u not implemented\n", filter->dbch_devicetype );
+    return TRUE;
+}
+
 static DWORD WINAPI device_notify_proc( void *arg )
 {
     WCHAR endpoint[] = L"\\pipe\\wine_plugplay";
-    WCHAR protseq[] = L"ncalrpc";
+    WCHAR protseq[] = L"ncacn_np";
     RPC_WSTR binding_str;
     DWORD err = ERROR_SUCCESS;
     struct device_notify_registration *registration;
+    struct device_notification_details *details_copy;
+    unsigned int details_copy_nelems, details_copy_size;
     plugplay_rpc_handle handle = NULL;
     DWORD code = 0;
-    unsigned int size;
+    unsigned int i, size;
     BYTE *buf;
 
     if ((err = RpcStringBindingComposeW( NULL, protseq, NULL, endpoint, NULL, &binding_str )))
@@ -2026,6 +2050,9 @@ static DWORD WINAPI device_notify_proc( void *arg )
         return 1;
     }
 
+    details_copy_size = 8;
+    details_copy = heap_alloc( details_copy_size * sizeof(*details_copy) );
+
     for (;;)
     {
         buf = NULL;
@@ -2046,15 +2073,32 @@ static DWORD WINAPI device_notify_proc( void *arg )
             break;
         }
 
+        /* Make a copy to avoid a hang if a callback tries to register or unregister for notifications. */
+        i = 0;
+        details_copy_nelems = 0;
         EnterCriticalSection( &service_cs );
         LIST_FOR_EACH_ENTRY(registration, &device_notify_list, struct device_notify_registration, entry)
         {
-            registration->details.cb( registration->details.handle, code, (DEV_BROADCAST_HDR *)buf );
+            details_copy[i++] = registration->details;
+            details_copy_nelems++;
+            if (i == details_copy_size)
+            {
+                details_copy_size *= 2;
+                details_copy = heap_realloc( details_copy, details_copy_size * sizeof(*details_copy) );
+            }
         }
         LeaveCriticalSection(&service_cs);
+
+        for (i = 0; i < details_copy_nelems; i++)
+        {
+            if (!notification_filter_matches( &details_copy[i].filter.header, (DEV_BROADCAST_HDR *)buf )) continue;
+            details_copy[i].cb( details_copy[i].handle, code, (DEV_BROADCAST_HDR *)buf );
+        }
         MIDL_user_free(buf);
     }
 
+    heap_free( details_copy );
+
     __TRY
     {
         plugplay_unregister_listener( handle );
@@ -2078,8 +2122,6 @@ HDEVNOTIFY WINAPI I_ScRegisterDeviceNotification( struct device_notification_det
 
     TRACE("callback %p, handle %p, filter %p, flags %#x\n", details->cb, details->handle, filter, flags);
 
-    if (filter) FIXME("Notification filters are not yet implemented.\n");
-
     if (!(registration = heap_alloc(sizeof(struct device_notify_registration))))
     {
         SetLastError(ERROR_NOT_ENOUGH_MEMORY);
