@@ -246,12 +246,7 @@ static LRESULT DEFWND_DefWinProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
         return NC_HandleNCPaint( hwnd, (HRGN)wParam );
 
     case WM_NCMOUSEMOVE:
-        {
-            POINT pt;
-            pt.x = (short)LOWORD(lParam);
-            pt.y = (short)HIWORD(lParam);
-            return NC_HandleNCMouseMove( hwnd, pt );
-        }
+        return NC_HandleNCMouseMove( hwnd, wParam, lParam );
 
     case WM_NCMOUSELEAVE:
         return NC_HandleNCMouseLeave( hwnd );
@@ -322,7 +317,7 @@ static LRESULT DEFWND_DefWinProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
 
     case WM_CONTEXTMENU:
         if (GetWindowLongW( hwnd, GWL_STYLE ) & WS_CHILD)
-            SendMessageW( GetParent(hwnd), msg, wParam, lParam );
+            SendMessageW( GetParent(hwnd), msg, (WPARAM)hwnd, lParam );
         else
         {
             LONG hitcode;
