@@ -77,6 +77,43 @@ static WORD get_key_state(void)
 }
 
 
+/***********************************************************************
+ *           get_locale_kbd_layout
+ */
+static HKL get_locale_kbd_layout(void)
+{
+    ULONG_PTR layout;
+    LANGID langid;
+
+    /* FIXME:
+     *
+     * layout = main_key_tab[kbd_layout].lcid;
+     *
+     * Winword uses return value of GetKeyboardLayout as a codepage
+     * to translate ANSI keyboard messages to unicode. But we have
+     * a problem with it: for instance Polish keyboard layout is
+     * identical to the US one, and therefore instead of the Polish
+     * locale id we return the US one.
+     */
+
+    layout = GetUserDefaultLCID();
+
+    /*
+     * Microsoft Office expects this value to be something specific
+     * for Japanese and Korean Windows with an IME the value is 0xe001
+     * We should probably check to see if an IME exists and if so then
+     * set this word properly.
+     */
+    langid = PRIMARYLANGID( LANGIDFROMLCID( layout ) );
+    if (langid == LANG_CHINESE || langid == LANG_JAPANESE || langid == LANG_KOREAN)
+        layout = MAKELONG( layout, 0xe001 ); /* IME */
+    else
+        layout = MAKELONG( layout, layout );
+
+    return (HKL)layout;
+}
+
+
 /**********************************************************************
  *		set_capture_window
  */
@@ -139,17 +176,15 @@ static void update_mouse_coords( INPUT *input )
     if (input->u.mi.dwFlags & MOUSEEVENTF_ABSOLUTE)
     {
         DPI_AWARENESS_CONTEXT context = SetThreadDpiAwarenessContext( DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE );
+        RECT rc;
+
         if (input->u.mi.dwFlags & MOUSEEVENTF_VIRTUALDESK)
-        {
-            RECT rc = get_virtual_screen_rect();
-            input->u.mi.dx = rc.left + ((input->u.mi.dx * (rc.right - rc.left)) >> 16);
-            input->u.mi.dy = rc.top  + ((input->u.mi.dy * (rc.bottom - rc.top)) >> 16);
-        }
+            rc = get_virtual_screen_rect();
         else
-        {
-            input->u.mi.dx = (input->u.mi.dx * GetSystemMetrics( SM_CXSCREEN )) >> 16;
-            input->u.mi.dy = (input->u.mi.dy * GetSystemMetrics( SM_CYSCREEN )) >> 16;
-        }
+            rc = get_primary_monitor_rect();
+
+        input->u.mi.dx = rc.left + ((input->u.mi.dx * (rc.right - rc.left)) >> 16);
+        input->u.mi.dy = rc.top  + ((input->u.mi.dy * (rc.bottom - rc.top)) >> 16);
         SetThreadDpiAwarenessContext( context );
     }
     else
@@ -539,90 +574,6 @@ BOOL WINAPI GetLastInputInfo(PLASTINPUTINFO plii)
 }
 
 
-/**********************************************************************
- *		AttachThreadInput (USER32.@)
- *
- * Attaches the input processing mechanism of one thread to that of
- * another thread.
- */
-BOOL WINAPI AttachThreadInput( DWORD from, DWORD to, BOOL attach )
-{
-    BOOL ret;
-
-    SERVER_START_REQ( attach_thread_input )
-    {
-        req->tid_from = from;
-        req->tid_to   = to;
-        req->attach   = attach;
-        ret = !wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
-    return ret;
-}
-
-
-/**********************************************************************
- *		GetKeyState (USER32.@)
- *
- * An application calls the GetKeyState function in response to a
- * keyboard-input message.  This function retrieves the state of the key
- * at the time the input message was generated.
- */
-SHORT WINAPI DECLSPEC_HOTPATCH GetKeyState(INT vkey)
-{
-    SHORT retval = 0;
-
-    SERVER_START_REQ( get_key_state )
-    {
-        req->key = vkey;
-        if (!wine_server_call( req )) retval = (signed char)(reply->state & 0x81);
-    }
-    SERVER_END_REQ;
-    TRACE("key (0x%x) -> %x\n", vkey, retval);
-    return retval;
-}
-
-
-/**********************************************************************
- *		GetKeyboardState (USER32.@)
- */
-BOOL WINAPI DECLSPEC_HOTPATCH GetKeyboardState( LPBYTE state )
-{
-    BOOL ret;
-    UINT i;
-
-    TRACE("(%p)\n", state);
-
-    memset( state, 0, 256 );
-    SERVER_START_REQ( get_key_state )
-    {
-        req->key = -1;
-        wine_server_set_reply( req, state, 256 );
-        ret = !wine_server_call_err( req );
-        for (i = 0; i < 256; i++) state[i] &= 0x81;
-    }
-    SERVER_END_REQ;
-    return ret;
-}
-
-
-/**********************************************************************
- *		SetKeyboardState (USER32.@)
- */
-BOOL WINAPI SetKeyboardState( LPBYTE state )
-{
-    BOOL ret;
-
-    SERVER_START_REQ( set_key_state )
-    {
-        wine_server_add_data( req, state, 256 );
-        ret = !wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
-    return ret;
-}
-
-
 /**********************************************************************
  *		VkKeyScanA (USER32.@)
  *
@@ -655,7 +606,7 @@ SHORT WINAPI VkKeyScanA(CHAR cChar)
  */
 SHORT WINAPI VkKeyScanW(WCHAR cChar)
 {
-    return VkKeyScanExW(cChar, GetKeyboardLayout(0));
+    return NtUserVkKeyScanEx( cChar, NtUserGetKeyboardLayout(0) );
 }
 
 /**********************************************************************
@@ -668,15 +619,7 @@ WORD WINAPI VkKeyScanExA(CHAR cChar, HKL dwhkl)
     if (IsDBCSLeadByte(cChar)) return -1;
 
     MultiByteToWideChar(CP_ACP, 0, &cChar, 1, &wChar, 1);
-    return VkKeyScanExW(wChar, dwhkl);
-}
-
-/******************************************************************************
- *		VkKeyScanExW (USER32.@)
- */
-WORD WINAPI VkKeyScanExW(WCHAR cChar, HKL dwhkl)
-{
-    return USER_Driver->pVkKeyScanEx(cChar, dwhkl);
+    return NtUserVkKeyScanEx( wChar, dwhkl );
 }
 
 /**********************************************************************
@@ -706,7 +649,7 @@ DWORD WINAPI OemKeyScan( WORD oem )
 INT WINAPI GetKeyboardType(INT nTypeFlag)
 {
     TRACE_(keyboard)("(%d)\n", nTypeFlag);
-    if (LOWORD(GetKeyboardLayout(0)) == MAKELANGID(LANG_JAPANESE, SUBLANG_JAPANESE_JAPAN))
+    if (LOWORD(NtUserGetKeyboardLayout(0)) == MAKELANGID(LANG_JAPANESE, SUBLANG_JAPANESE_JAPAN))
     {
         /* scan code for `_', the key left of r-shift, in Japanese 106 keyboard */
         const UINT JP106_VSC_USCORE = 0x73;
@@ -747,7 +690,7 @@ INT WINAPI GetKeyboardType(INT nTypeFlag)
  */
 UINT WINAPI MapVirtualKeyA(UINT code, UINT maptype)
 {
-    return MapVirtualKeyExA( code, maptype, GetKeyboardLayout(0) );
+    return MapVirtualKeyExA( code, maptype, NtUserGetKeyboardLayout(0) );
 }
 
 /******************************************************************************
@@ -755,7 +698,7 @@ UINT WINAPI MapVirtualKeyA(UINT code, UINT maptype)
  */
 UINT WINAPI MapVirtualKeyW(UINT code, UINT maptype)
 {
-    return MapVirtualKeyExW(code, maptype, GetKeyboardLayout(0));
+    return NtUserMapVirtualKeyEx( code, maptype, NtUserGetKeyboardLayout(0) );
 }
 
 /******************************************************************************
@@ -765,7 +708,7 @@ UINT WINAPI MapVirtualKeyExA(UINT code, UINT maptype, HKL hkl)
 {
     UINT ret;
 
-    ret = MapVirtualKeyExW( code, maptype, hkl );
+    ret = NtUserMapVirtualKeyEx( code, maptype, hkl );
     if (maptype == MAPVK_VK_TO_CHAR)
     {
         BYTE ch = 0;
@@ -777,16 +720,6 @@ UINT WINAPI MapVirtualKeyExA(UINT code, UINT maptype, HKL hkl)
     return ret;
 }
 
-/******************************************************************************
- *		MapVirtualKeyExW (USER32.@)
- */
-UINT WINAPI MapVirtualKeyExW(UINT code, UINT maptype, HKL hkl)
-{
-    TRACE_(keyboard)("(%X, %d, %p)\n", code, maptype, hkl);
-
-    return USER_Driver->pMapVirtualKeyEx(code, maptype, hkl);
-}
-
 /****************************************************************************
  *		GetKBCodePage (USER32.@)
  */
@@ -795,18 +728,6 @@ UINT WINAPI GetKBCodePage(void)
     return GetOEMCP();
 }
 
-/***********************************************************************
- *		GetKeyboardLayout (USER32.@)
- *
- *        - device handle for keyboard layout defaulted to
- *          the language id. This is the way Windows default works.
- *        - the thread identifier is also ignored.
- */
-HKL WINAPI GetKeyboardLayout(DWORD thread_id)
-{
-    return USER_Driver->pGetKeyboardLayout(thread_id);
-}
-
 /****************************************************************************
  *		GetKeyboardLayoutNameA (USER32.@)
  */
@@ -814,24 +735,11 @@ BOOL WINAPI GetKeyboardLayoutNameA(LPSTR pszKLID)
 {
     WCHAR buf[KL_NAMELENGTH];
 
-    if (GetKeyboardLayoutNameW(buf))
+    if (NtUserGetKeyboardLayoutName( buf ))
         return WideCharToMultiByte( CP_ACP, 0, buf, -1, pszKLID, KL_NAMELENGTH, NULL, NULL ) != 0;
     return FALSE;
 }
 
-/****************************************************************************
- *		GetKeyboardLayoutNameW (USER32.@)
- */
-BOOL WINAPI GetKeyboardLayoutNameW(LPWSTR pwszKLID)
-{
-    if (!pwszKLID)
-    {
-        SetLastError(ERROR_NOACCESS);
-        return FALSE;
-    }
-    return USER_Driver->pGetKeyboardLayoutName(pwszKLID);
-}
-
 /****************************************************************************
  *		GetKeyNameTextA (USER32.@)
  */
@@ -840,7 +748,7 @@ INT WINAPI GetKeyNameTextA(LONG lParam, LPSTR lpBuffer, INT nSize)
     WCHAR buf[256];
     INT ret;
 
-    if (!nSize || !GetKeyNameTextW(lParam, buf, 256))
+    if (!nSize || !NtUserGetKeyNameText( lParam, buf, ARRAYSIZE(buf) ))
     {
         lpBuffer[0] = 0;
         return 0;
@@ -856,32 +764,12 @@ INT WINAPI GetKeyNameTextA(LONG lParam, LPSTR lpBuffer, INT nSize)
     return ret;
 }
 
-/****************************************************************************
- *		GetKeyNameTextW (USER32.@)
- */
-INT WINAPI GetKeyNameTextW(LONG lParam, LPWSTR lpBuffer, INT nSize)
-{
-    if (!lpBuffer || !nSize) return 0;
-    return USER_Driver->pGetKeyNameText( lParam, lpBuffer, nSize );
-}
-
 /****************************************************************************
  *		ToUnicode (USER32.@)
  */
-INT WINAPI ToUnicode(UINT virtKey, UINT scanCode, const BYTE *lpKeyState,
-		     LPWSTR lpwStr, int size, UINT flags)
+INT WINAPI ToUnicode( UINT virt, UINT scan, const BYTE *state, LPWSTR str, int size, UINT flags )
 {
-    return ToUnicodeEx(virtKey, scanCode, lpKeyState, lpwStr, size, flags, GetKeyboardLayout(0));
-}
-
-/****************************************************************************
- *		ToUnicodeEx (USER32.@)
- */
-INT WINAPI ToUnicodeEx(UINT virtKey, UINT scanCode, const BYTE *lpKeyState,
-		       LPWSTR lpwStr, int size, UINT flags, HKL hkl)
-{
-    if (!lpKeyState) return 0;
-    return USER_Driver->pToUnicodeEx(virtKey, scanCode, lpKeyState, lpwStr, size, flags, hkl);
+    return NtUserToUnicodeEx( virt, scan, state, str, size, flags, NtUserGetKeyboardLayout(0) );
 }
 
 /****************************************************************************
@@ -890,7 +778,8 @@ INT WINAPI ToUnicodeEx(UINT virtKey, UINT scanCode, const BYTE *lpKeyState,
 INT WINAPI ToAscii( UINT virtKey, UINT scanCode, const BYTE *lpKeyState,
                     LPWORD lpChar, UINT flags )
 {
-    return ToAsciiEx(virtKey, scanCode, lpKeyState, lpChar, flags, GetKeyboardLayout(0));
+    return ToAsciiEx( virtKey, scanCode, lpKeyState, lpChar, flags,
+                      NtUserGetKeyboardLayout(0) );
 }
 
 /****************************************************************************
@@ -902,23 +791,13 @@ INT WINAPI ToAsciiEx( UINT virtKey, UINT scanCode, const BYTE *lpKeyState,
     WCHAR uni_chars[2];
     INT ret, n_ret;
 
-    ret = ToUnicodeEx(virtKey, scanCode, lpKeyState, uni_chars, 2, flags, dwhkl);
+    ret = NtUserToUnicodeEx( virtKey, scanCode, lpKeyState, uni_chars, 2, flags, dwhkl );
     if (ret < 0) n_ret = 1; /* FIXME: make ToUnicode return 2 for dead chars */
     else n_ret = ret;
     WideCharToMultiByte(CP_ACP, 0, uni_chars, n_ret, (LPSTR)lpChar, 2, NULL, NULL);
     return ret;
 }
 
-/**********************************************************************
- *		ActivateKeyboardLayout (USER32.@)
- */
-HKL WINAPI ActivateKeyboardLayout(HKL hLayout, UINT flags)
-{
-    TRACE_(keyboard)("(%p, %d)\n", hLayout, flags);
-
-    return USER_Driver->pActivateKeyboardLayout(hLayout, flags);
-}
-
 /**********************************************************************
  *		BlockInput (USER32.@)
  */
@@ -930,19 +809,6 @@ BOOL WINAPI BlockInput(BOOL fBlockIt)
     return FALSE;
 }
 
-/***********************************************************************
- *		GetKeyboardLayoutList (USER32.@)
- *
- * Return number of values available if either input parm is
- *  0, per MS documentation.
- */
-UINT WINAPI GetKeyboardLayoutList(INT nBuff, HKL *layouts)
-{
-    TRACE_(keyboard)( "(%d, %p)\n", nBuff, layouts );
-
-    return USER_Driver->pGetKeyboardLayoutList( nBuff, layouts );
-}
-
 
 /***********************************************************************
  *		RegisterHotKey (USER32.@)
@@ -979,42 +845,38 @@ BOOL WINAPI RegisterHotKey(HWND hwnd,INT id,UINT modifiers,UINT vk)
     return ret;
 }
 
-/***********************************************************************
- *		UnregisterHotKey (USER32.@)
- */
-BOOL WINAPI UnregisterHotKey(HWND hwnd,INT id)
-{
-    BOOL ret;
-    UINT modifiers, vk;
-
-    TRACE_(keyboard)("(%p,%d)\n",hwnd,id);
-
-    SERVER_START_REQ( unregister_hotkey )
-    {
-        req->window = wine_server_user_handle( hwnd );
-        req->id = id;
-        if ((ret = !wine_server_call_err( req )))
-        {
-            modifiers = reply->flags;
-            vk = reply->vkey;
-        }
-    }
-    SERVER_END_REQ;
-
-    if (ret)
-        USER_Driver->pUnregisterHotKey(hwnd, modifiers, vk);
-
-    return ret;
-}
-
 /***********************************************************************
  *		LoadKeyboardLayoutW (USER32.@)
  */
-HKL WINAPI LoadKeyboardLayoutW(LPCWSTR pwszKLID, UINT Flags)
+HKL WINAPI LoadKeyboardLayoutW( const WCHAR *name, UINT flags )
 {
-    TRACE_(keyboard)("(%s, %d)\n", debugstr_w(pwszKLID), Flags);
+    WCHAR layout_path[MAX_PATH], value[5];
+    DWORD value_size, tmp;
+    HKEY hkey;
+    HKL layout;
 
-    return USER_Driver->pLoadKeyboardLayout(pwszKLID, Flags);
+    FIXME_(keyboard)( "name %s, flags %x, semi-stub!\n", debugstr_w( name ), flags );
+
+    tmp = wcstoul( name, NULL, 16 );
+    if (HIWORD( tmp )) layout = UlongToHandle( tmp );
+    else layout = UlongToHandle( MAKELONG( LOWORD( tmp ), LOWORD( tmp ) ) );
+
+    wcscpy( layout_path, L"System\\CurrentControlSet\\Control\\Keyboard Layouts\\" );
+    wcscat( layout_path, name );
+
+    if (!RegOpenKeyW( HKEY_LOCAL_MACHINE, layout_path, &hkey ))
+    {
+        value_size = sizeof(value);
+        if (!RegGetValueW( hkey, NULL, L"Layout Id", RRF_RT_REG_SZ, NULL, (void *)&value, &value_size ))
+            layout = UlongToHandle( MAKELONG( LOWORD( tmp ), 0xf000 | (wcstoul( value, NULL, 16 ) & 0xfff) ) );
+
+        RegCloseKey( hkey );
+    }
+
+    if ((flags & KLF_ACTIVATE) && NtUserActivateKeyboardLayout( layout, 0 )) return layout;
+
+    /* FIXME: semi-stub: returning default layout */
+    return get_locale_kbd_layout();
 }
 
 /***********************************************************************
@@ -1037,11 +899,11 @@ HKL WINAPI LoadKeyboardLayoutA(LPCSTR pwszKLID, UINT Flags)
 /***********************************************************************
  *		UnloadKeyboardLayout (USER32.@)
  */
-BOOL WINAPI UnloadKeyboardLayout(HKL hkl)
+BOOL WINAPI UnloadKeyboardLayout( HKL layout )
 {
-    TRACE_(keyboard)("(%p)\n", hkl);
-
-    return USER_Driver->pUnloadKeyboardLayout(hkl);
+    FIXME_(keyboard)( "layout %p, stub!\n", layout );
+    SetLastError( ERROR_CALL_NOT_IMPLEMENTED );
+    return FALSE;
 }
 
 typedef struct __TRACKINGLIST {
@@ -1281,73 +1143,6 @@ TrackMouseEvent (TRACKMOUSEEVENT *ptme)
     return TRUE;
 }
 
-/***********************************************************************
- * GetMouseMovePointsEx [USER32]
- *
- * RETURNS
- *     Success: count of point set in the buffer
- *     Failure: -1
- */
-int WINAPI GetMouseMovePointsEx( UINT size, LPMOUSEMOVEPOINT ptin, LPMOUSEMOVEPOINT ptout, int count, DWORD resolution )
-{
-    cursor_pos_t *pos, positions[64];
-    int copied;
-    unsigned int i;
-
-
-    TRACE( "%d, %p, %p, %d, %d\n", size, ptin, ptout, count, resolution );
-
-    if ((size != sizeof(MOUSEMOVEPOINT)) || (count < 0) || (count > ARRAY_SIZE( positions )))
-    {
-        SetLastError( ERROR_INVALID_PARAMETER );
-        return -1;
-    }
-
-    if (!ptin || (!ptout && count))
-    {
-        SetLastError( ERROR_NOACCESS );
-        return -1;
-    }
-
-    if (resolution != GMMP_USE_DISPLAY_POINTS)
-    {
-        FIXME( "only GMMP_USE_DISPLAY_POINTS is supported for now\n" );
-        SetLastError( ERROR_POINT_NOT_FOUND );
-        return -1;
-    }
-
-    SERVER_START_REQ( get_cursor_history )
-    {
-        wine_server_set_reply( req, &positions, sizeof(positions) );
-        if (wine_server_call_err( req )) return -1;
-    }
-    SERVER_END_REQ;
-
-    for (i = 0; i < ARRAY_SIZE( positions ); i++)
-    {
-        pos = &positions[i];
-        if (ptin->x == pos->x && ptin->y == pos->y && (!ptin->time || ptin->time == pos->time))
-            break;
-    }
-
-    if (i == ARRAY_SIZE( positions ))
-    {
-        SetLastError( ERROR_POINT_NOT_FOUND );
-        return -1;
-    }
-
-    for (copied = 0; copied < count && i < ARRAY_SIZE( positions ); copied++, i++)
-    {
-        pos = &positions[i];
-        ptout[copied].x = pos->x;
-        ptout[copied].y = pos->y;
-        ptout[copied].time = pos->time;
-        ptout[copied].dwExtraInfo = pos->info;
-    }
-
-    return copied;
-}
-
 /***********************************************************************
  *		EnableMouseInPointer (USER32.@)
  */
@@ -1375,6 +1170,11 @@ struct device_notification_details
 {
     DWORD (CALLBACK *cb)(HANDLE handle, DWORD flags, DEV_BROADCAST_HDR *header);
     HANDLE handle;
+    union
+    {
+        DEV_BROADCAST_HDR header;
+        DEV_BROADCAST_DEVICEINTERFACE_W iface;
+    } filter;
 };
 
 extern HDEVNOTIFY WINAPI I_ScRegisterDeviceNotification( struct device_notification_details *details,
@@ -1386,13 +1186,9 @@ extern BOOL WINAPI I_ScUnregisterDeviceNotification( HDEVNOTIFY handle );
  *
  * See RegisterDeviceNotificationW.
  */
-HDEVNOTIFY WINAPI RegisterDeviceNotificationA(HANDLE hRecipient, LPVOID pNotificationFilter, DWORD dwFlags)
+HDEVNOTIFY WINAPI RegisterDeviceNotificationA( HANDLE handle, void *filter, DWORD flags )
 {
-    TRACE("(hwnd=%p, filter=%p,flags=0x%08x)\n",
-        hRecipient,pNotificationFilter,dwFlags);
-    if (pNotificationFilter)
-        FIXME("The notification filter will requires an A->W when filter support is implemented\n");
-    return RegisterDeviceNotificationW(hRecipient, pNotificationFilter, dwFlags);
+    return RegisterDeviceNotificationW( handle, filter, flags );
 }
 
 /***********************************************************************
@@ -1401,11 +1197,43 @@ HDEVNOTIFY WINAPI RegisterDeviceNotificationA(HANDLE hRecipient, LPVOID pNotific
 HDEVNOTIFY WINAPI RegisterDeviceNotificationW( HANDLE handle, void *filter, DWORD flags )
 {
     struct device_notification_details details;
+    DEV_BROADCAST_HDR *header = filter;
 
     TRACE("handle %p, filter %p, flags %#x\n", handle, filter, flags);
 
     if (flags & ~(DEVICE_NOTIFY_SERVICE_HANDLE | DEVICE_NOTIFY_ALL_INTERFACE_CLASSES))
-        FIXME("unhandled flags %#x\n", flags);
+    {
+        SetLastError( ERROR_INVALID_PARAMETER );
+        return NULL;
+    }
+
+    if (!(flags & DEVICE_NOTIFY_SERVICE_HANDLE) && !IsWindow( handle ))
+    {
+        SetLastError( ERROR_INVALID_PARAMETER );
+        return NULL;
+    }
+
+    if (!header) details.filter.header.dbch_devicetype = 0;
+    else if (header->dbch_devicetype == DBT_DEVTYP_DEVICEINTERFACE)
+    {
+        DEV_BROADCAST_DEVICEINTERFACE_W *iface = (DEV_BROADCAST_DEVICEINTERFACE_W *)header;
+        details.filter.iface = *iface;
+
+        if (flags & DEVICE_NOTIFY_ALL_INTERFACE_CLASSES)
+            details.filter.iface.dbcc_size = offsetof( DEV_BROADCAST_DEVICEINTERFACE_W, dbcc_classguid );
+        else
+            details.filter.iface.dbcc_size = offsetof( DEV_BROADCAST_DEVICEINTERFACE_W, dbcc_name );
+    }
+    else if (header->dbch_devicetype == DBT_DEVTYP_HANDLE)
+    {
+        FIXME( "DBT_DEVTYP_HANDLE filter type not implemented\n" );
+        details.filter.header.dbch_devicetype = 0;
+    }
+    else
+    {
+        SetLastError( ERROR_INVALID_DATA );
+        return NULL;
+    }
 
     details.handle = handle;
 
