@@ -607,13 +607,28 @@ BOOL wined3d_buffer_load_location(struct wined3d_buffer *buffer,
     switch (location)
     {
         case WINED3D_LOCATION_SYSMEM:
-            range.offset = 0;
-            range.size = buffer->resource.size;
-            buffer->buffer_ops->buffer_download_ranges(buffer, context,
-                    buffer->resource.heap_memory, 0, 1, &range);
+            if (buffer->locations & WINED3D_LOCATION_CLEARED)
+            {
+                memset(buffer->resource.heap_memory, 0, buffer->resource.size);
+            }
+            else
+            {
+                range.offset = 0;
+                range.size = buffer->resource.size;
+                buffer->buffer_ops->buffer_download_ranges(buffer, context,
+                        buffer->resource.heap_memory, 0, 1, &range);
+            }
             break;
 
         case WINED3D_LOCATION_BUFFER:
+            if (buffer->locations & WINED3D_LOCATION_CLEARED)
+            {
+                /* FIXME: Clear the buffer on the GPU if possible. */
+                if (!wined3d_buffer_prepare_location(buffer, context, WINED3D_LOCATION_SYSMEM))
+                    return FALSE;
+                memset(buffer->resource.heap_memory, 0, buffer->resource.size);
+            }
+
             if (!buffer->conversion_map)
                 buffer->buffer_ops->buffer_upload_ranges(buffer, context,
                         buffer->resource.heap_memory, 0, buffer->modified_areas, buffer->maps);
@@ -650,17 +665,15 @@ DWORD wined3d_buffer_get_memory(struct wined3d_buffer *buffer, struct wined3d_co
     TRACE("buffer %p, context %p, data %p, locations %s.\n",
             buffer, context, data, wined3d_debug_location(locations));
 
-    if (locations & WINED3D_LOCATION_DISCARDED)
+    if (locations & (WINED3D_LOCATION_DISCARDED | WINED3D_LOCATION_CLEARED))
     {
         locations = ((buffer->flags & WINED3D_BUFFER_USE_BO) ? WINED3D_LOCATION_BUFFER : WINED3D_LOCATION_SYSMEM);
-        if (!wined3d_buffer_prepare_location(buffer, context, locations))
+        if (!wined3d_buffer_load_location(buffer, context, locations))
         {
             data->buffer_object = 0;
             data->addr = NULL;
             return 0;
         }
-        wined3d_buffer_validate_location(buffer, locations);
-        wined3d_buffer_invalidate_location(buffer, WINED3D_LOCATION_DISCARDED);
     }
     if (locations & WINED3D_LOCATION_BUFFER)
     {
@@ -913,105 +926,110 @@ static HRESULT buffer_resource_sub_resource_map(struct wined3d_resource *resourc
         void **map_ptr, const struct wined3d_box *box, uint32_t flags)
 {
     struct wined3d_buffer *buffer = buffer_from_resource(resource);
+    unsigned int offset, size, dirty_offset, dirty_size;
     struct wined3d_device *device = resource->device;
     struct wined3d_context *context;
-    unsigned int offset, size;
+    struct wined3d_bo_address addr;
     uint8_t *base;
     LONG count;
 
     TRACE("resource %p, sub_resource_idx %u, map_ptr %p, box %s, flags %#x.\n",
             resource, sub_resource_idx, map_ptr, debug_box(box), flags);
 
-    offset = box->left;
-    size = box->right - box->left;
+    dirty_offset = offset = box->left;
+    dirty_size = size = box->right - box->left;
 
     count = ++resource->map_count;
 
-    if (buffer->buffer_object)
+    /* DISCARD invalidates the entire buffer, regardless of the specified
+     * offset and size. Some applications also depend on the entire buffer
+     * being uploaded in that case. Two such applications are Port Royale
+     * and Darkstar One. */
+    if (flags & WINED3D_MAP_DISCARD)
     {
-        unsigned int dirty_offset = offset, dirty_size = size;
-        struct wined3d_bo_address addr;
+        dirty_offset = 0;
+        dirty_size = 0;
+    }
 
-        /* DISCARD invalidates the entire buffer, regardless of the specified
-         * offset and size. Some applications also depend on the entire buffer
-         * being uploaded in that case. Two such applications are Port Royale
-         * and Darkstar One. */
-        if (flags & WINED3D_MAP_DISCARD)
+    if (((flags & WINED3D_MAP_WRITE) && !(flags & (WINED3D_MAP_NOOVERWRITE | WINED3D_MAP_DISCARD)))
+            || (!(flags & WINED3D_MAP_WRITE) && (buffer->locations & WINED3D_LOCATION_SYSMEM))
+            || buffer->flags & WINED3D_BUFFER_PIN_SYSMEM
+            || !(buffer->flags & WINED3D_BUFFER_USE_BO))
+    {
+        if (!(buffer->locations & WINED3D_LOCATION_SYSMEM))
         {
-            dirty_offset = 0;
-            dirty_size = 0;
+            context = context_acquire(device, NULL, 0);
+            wined3d_buffer_load_location(buffer, context, WINED3D_LOCATION_SYSMEM);
+            context_release(context);
         }
 
-        if (((flags & WINED3D_MAP_WRITE) && !(flags & (WINED3D_MAP_NOOVERWRITE | WINED3D_MAP_DISCARD)))
-                || (!(flags & WINED3D_MAP_WRITE) && (buffer->locations & WINED3D_LOCATION_SYSMEM))
-                || buffer->flags & WINED3D_BUFFER_PIN_SYSMEM)
-        {
-            if (!(buffer->locations & WINED3D_LOCATION_SYSMEM))
-            {
-                context = context_acquire(device, NULL, 0);
-                wined3d_buffer_load_location(buffer, context, WINED3D_LOCATION_SYSMEM);
-                context_release(context);
-            }
+        if (flags & WINED3D_MAP_WRITE)
+            wined3d_buffer_invalidate_range(buffer, ~WINED3D_LOCATION_SYSMEM, dirty_offset, dirty_size);
+    }
+    else
+    {
+        context = context_acquire(device, NULL, 0);
 
-            if (flags & WINED3D_MAP_WRITE)
-                wined3d_buffer_invalidate_range(buffer, WINED3D_LOCATION_BUFFER, dirty_offset, dirty_size);
+        if (flags & WINED3D_MAP_DISCARD)
+        {
+            if (!wined3d_buffer_prepare_location(buffer, context, WINED3D_LOCATION_BUFFER))
+            {
+                context_release(context);
+                return E_OUTOFMEMORY;
+            }
+            wined3d_buffer_validate_location(buffer, WINED3D_LOCATION_BUFFER);
         }
         else
         {
-            context = context_acquire(device, NULL, 0);
-
-            if (flags & WINED3D_MAP_DISCARD)
-                wined3d_buffer_validate_location(buffer, WINED3D_LOCATION_BUFFER);
-            else
-                wined3d_buffer_load_location(buffer, context, WINED3D_LOCATION_BUFFER);
-
-            if (flags & WINED3D_MAP_WRITE)
-            {
-                wined3d_buffer_invalidate_location(buffer, WINED3D_LOCATION_SYSMEM);
-                buffer_invalidate_bo_range(buffer, dirty_offset, dirty_size);
-            }
-
-            if ((flags & WINED3D_MAP_DISCARD) && resource->heap_memory)
-                wined3d_buffer_evict_sysmem(buffer);
-
-            if (count == 1)
-            {
-                addr.buffer_object = buffer->buffer_object;
-                addr.addr = 0;
-                buffer->map_ptr = wined3d_context_map_bo_address(context, &addr, resource->size, flags);
-                /* We are accessing buffer->resource.client from the CS thread,
-                 * but it's safe because the client thread will wait for the
-                 * map to return, thus completely serializing this call with
-                 * other client code. */
-                buffer->resource.client.addr = addr;
-
-                if (((DWORD_PTR)buffer->map_ptr) & (RESOURCE_ALIGNMENT - 1))
-                {
-                    WARN("Pointer %p is not %u byte aligned.\n", buffer->map_ptr, RESOURCE_ALIGNMENT);
-
-                    wined3d_context_unmap_bo_address(context, &addr, 0, NULL);
-                    buffer->map_ptr = NULL;
-
-                    if (resource->usage & WINED3DUSAGE_DYNAMIC)
-                    {
-                        /* The extra copy is more expensive than not using VBOs
-                         * at all on the NVIDIA Linux driver, which is the
-                         * only driver that returns unaligned pointers. */
-                        TRACE("Dynamic buffer, dropping VBO.\n");
-                        wined3d_buffer_drop_bo(buffer);
-                    }
-                    else
-                    {
-                        TRACE("Falling back to doublebuffered operation.\n");
-                        wined3d_buffer_load_location(buffer, context, WINED3D_LOCATION_SYSMEM);
-                        buffer->flags |= WINED3D_BUFFER_PIN_SYSMEM;
-                    }
-                    TRACE("New pointer is %p.\n", resource->heap_memory);
-                }
-            }
-
-            context_release(context);
+            wined3d_buffer_load_location(buffer, context, WINED3D_LOCATION_BUFFER);
         }
+
+        if (flags & WINED3D_MAP_WRITE)
+        {
+            wined3d_buffer_invalidate_location(buffer, ~WINED3D_LOCATION_BUFFER);
+            buffer_invalidate_bo_range(buffer, dirty_offset, dirty_size);
+        }
+
+        if ((flags & WINED3D_MAP_DISCARD) && resource->heap_memory)
+            wined3d_buffer_evict_sysmem(buffer);
+
+        if (count == 1)
+        {
+            addr.buffer_object = buffer->buffer_object;
+            addr.addr = 0;
+            buffer->map_ptr = wined3d_context_map_bo_address(context, &addr, resource->size, flags);
+            /* We are accessing buffer->resource.client from the CS thread,
+             * but it's safe because the client thread will wait for the
+             * map to return, thus completely serializing this call with
+             * other client code. */
+            buffer->resource.client.addr = addr;
+
+            if (((DWORD_PTR)buffer->map_ptr) & (RESOURCE_ALIGNMENT - 1))
+            {
+                WARN("Pointer %p is not %u byte aligned.\n", buffer->map_ptr, RESOURCE_ALIGNMENT);
+
+                wined3d_context_unmap_bo_address(context, &addr, 0, NULL);
+                buffer->map_ptr = NULL;
+
+                if (resource->usage & WINED3DUSAGE_DYNAMIC)
+                {
+                    /* The extra copy is more expensive than not using VBOs
+                     * at all on the NVIDIA Linux driver, which is the
+                     * only driver that returns unaligned pointers. */
+                    TRACE("Dynamic buffer, dropping VBO.\n");
+                    wined3d_buffer_drop_bo(buffer);
+                }
+                else
+                {
+                    TRACE("Falling back to doublebuffered operation.\n");
+                    wined3d_buffer_load_location(buffer, context, WINED3D_LOCATION_SYSMEM);
+                    buffer->flags |= WINED3D_BUFFER_PIN_SYSMEM;
+                }
+                TRACE("New pointer is %p.\n", resource->heap_memory);
+            }
+        }
+
+        context_release(context);
     }
 
     base = buffer->map_ptr ? buffer->map_ptr : resource->heap_memory;
@@ -1238,7 +1256,7 @@ static HRESULT wined3d_buffer_init(struct wined3d_buffer *buffer, struct wined3d
     }
     buffer->buffer_ops = buffer_ops;
     buffer->structure_byte_stride = desc->structure_byte_stride;
-    buffer->locations = data ? WINED3D_LOCATION_DISCARDED : WINED3D_LOCATION_SYSMEM;
+    buffer->locations = WINED3D_LOCATION_CLEARED;
 
     TRACE("buffer %p, size %#x, usage %#x, memory @ %p.\n",
             buffer, buffer->resource.size, buffer->resource.usage, buffer->resource.heap_memory);
@@ -1372,25 +1390,22 @@ static void wined3d_buffer_gl_unload_location(struct wined3d_buffer *buffer,
 static void wined3d_buffer_gl_upload_ranges(struct wined3d_buffer *buffer, struct wined3d_context *context,
         const void *data, unsigned int data_offset, unsigned int range_count, const struct wined3d_range *ranges)
 {
-    struct wined3d_context_gl *context_gl = wined3d_context_gl(context);
-    struct wined3d_bo_gl *bo_gl = wined3d_bo_gl(buffer->buffer_object);
-    struct wined3d_buffer_gl *buffer_gl = wined3d_buffer_gl(buffer);
-    const struct wined3d_gl_info *gl_info = context_gl->gl_info;
-    const struct wined3d_range *range;
+    struct wined3d_bo_address src, dst;
 
     TRACE("buffer %p, context %p, data %p, data_offset %u, range_count %u, ranges %p.\n",
             buffer, context, data, data_offset, range_count, ranges);
 
-    wined3d_buffer_gl_bind(buffer_gl, context_gl);
+    dst.buffer_object = buffer->buffer_object;
+    src.buffer_object = NULL;
 
     while (range_count--)
     {
-        range = &ranges[range_count];
-        GL_EXTCALL(glBufferSubData(bo_gl->binding, bo_gl->b.buffer_offset + range->offset,
-                range->size, (BYTE *)data + range->offset - data_offset));
+        const struct wined3d_range *range = &ranges[range_count];
+
+        src.addr = (uint8_t *)data + range->offset - data_offset;
+        dst.addr = (void *)(uintptr_t)range->offset;
+        wined3d_context_copy_bo_address(context, &dst, &src, range->size);
     }
-    wined3d_context_gl_reference_bo(context_gl, bo_gl);
-    checkGLcall("buffer upload");
 }
 
 /* Context activation is done by the caller. */
@@ -1501,6 +1516,7 @@ static BOOL wined3d_buffer_vk_create_buffer_object(struct wined3d_buffer_vk *buf
             vk_memory_type_from_access_flags(resource->access, resource->usage), bo_vk)))
     {
         WARN("Failed to create Vulkan buffer.\n");
+        heap_free(bo_vk);
         return FALSE;
     }
 
