@@ -23,18 +23,11 @@
 
 #include "config.h"
 #include <time.h>
-#include <stdarg.h>
 #include <stdlib.h>
 
-#include "ntstatus.h"
-#define WIN32_NO_STATUS
-#include "windef.h"
-#include "winbase.h"
-#include "winreg.h"
-#include "winuser.h"
-#include "winternl.h"
-
 #include "vulkan_private.h"
+#include "winreg.h"
+#include "ntuser.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(vulkan);
 
@@ -68,7 +61,6 @@ static uint32_t wine_vk_count_struct_(void *s, VkStructureType t)
 }
 
 static const struct vulkan_funcs *vk_funcs;
-static VkResult (*p_vkEnumerateInstanceVersion)(uint32_t *version);
 
 #define WINE_VK_ADD_DISPATCHABLE_MAPPING(instance, object, native_handle) \
     wine_vk_add_handle_mapping((instance), (uint64_t) (uintptr_t) (object), (uint64_t) (uintptr_t) (native_handle), &(object)->mapping)
@@ -122,9 +114,11 @@ static VkBool32 debug_utils_callback_conversion(VkDebugUtilsMessageSeverityFlagB
     const VkDebugUtilsMessengerCallbackDataEXT_host *callback_data,
     void *user_data)
 {
-    struct VkDebugUtilsMessengerCallbackDataEXT wine_callback_data;
+    struct wine_vk_debug_utils_params params;
     VkDebugUtilsObjectNameInfoEXT *object_name_infos;
     struct wine_debug_utils_messenger *object;
+    void *ret_ptr;
+    ULONG ret_len;
     VkBool32 result;
     unsigned int i;
 
@@ -138,12 +132,16 @@ static VkBool32 debug_utils_callback_conversion(VkDebugUtilsMessageSeverityFlagB
         return VK_FALSE;
     }
 
-    wine_callback_data = *((VkDebugUtilsMessengerCallbackDataEXT *) callback_data);
+    /* FIXME: we should pack all referenced structs instead of passing pointers */
+    params.user_callback = object->user_callback;
+    params.user_data = object->user_data;
+    params.severity = severity;
+    params.message_types = message_types;
+    params.data = *((VkDebugUtilsMessengerCallbackDataEXT *) callback_data);
 
-    object_name_infos = RtlAllocateHeap(GetProcessHeap(), HEAP_ZERO_MEMORY,
-                                        wine_callback_data.objectCount * sizeof(*object_name_infos));
+    object_name_infos = calloc(params.data.objectCount, sizeof(*object_name_infos));
 
-    for (i = 0; i < wine_callback_data.objectCount; i++)
+    for (i = 0; i < params.data.objectCount; i++)
     {
         object_name_infos[i].sType = callback_data->pObjects[i].sType;
         object_name_infos[i].pNext = callback_data->pObjects[i].pNext;
@@ -156,7 +154,7 @@ static VkBool32 debug_utils_callback_conversion(VkDebugUtilsMessageSeverityFlagB
             if (!object_name_infos[i].objectHandle)
             {
                 WARN("handle conversion failed 0x%s\n", wine_dbgstr_longlong(callback_data->pObjects[i].objectHandle));
-                RtlFreeHeap(GetProcessHeap(), 0, object_name_infos);
+                free(object_name_infos);
                 return VK_FALSE;
             }
         }
@@ -166,12 +164,13 @@ static VkBool32 debug_utils_callback_conversion(VkDebugUtilsMessageSeverityFlagB
         }
     }
 
-    wine_callback_data.pObjects = object_name_infos;
+    params.data.pObjects = object_name_infos;
 
     /* applications should always return VK_FALSE */
-    result = object->user_callback(severity, message_types, &wine_callback_data, object->user_data);
+    result = KeUserModeCallback( NtUserCallVulkanDebugUtilsCallback, &params, sizeof(params),
+                                 &ret_ptr, &ret_len );
 
-    RtlFreeHeap(GetProcessHeap(), 0, object_name_infos);
+    free(object_name_infos);
 
     return result;
 }
@@ -179,7 +178,10 @@ static VkBool32 debug_utils_callback_conversion(VkDebugUtilsMessageSeverityFlagB
 static VkBool32 debug_report_callback_conversion(VkDebugReportFlagsEXT flags, VkDebugReportObjectTypeEXT object_type,
     uint64_t object_handle, size_t location, int32_t code, const char *layer_prefix, const char *message, void *user_data)
 {
+    struct wine_vk_debug_report_params params;
     struct wine_debug_report_callback *object;
+    void *ret_ptr;
+    ULONG ret_len;
 
     TRACE("%#x, %#x, 0x%s, 0x%s, %d, %p, %p, %p\n", flags, object_type, wine_dbgstr_longlong(object_handle),
         wine_dbgstr_longlong(location), code, layer_prefix, message, user_data);
@@ -192,12 +194,22 @@ static VkBool32 debug_report_callback_conversion(VkDebugReportFlagsEXT flags, Vk
         return VK_FALSE;
     }
 
-    object_handle = wine_vk_get_wrapper(object->instance, object_handle);
-    if (!object_handle)
-        object_type = VK_DEBUG_REPORT_OBJECT_TYPE_UNKNOWN_EXT;
+    /* FIXME: we should pack all referenced structs instead of passing pointers */
+    params.user_callback = object->user_callback;
+    params.user_data = object->user_data;
+    params.flags = flags;
+    params.object_type = object_type;
+    params.location = location;
+    params.code = code;
+    params.layer_prefix = layer_prefix;
+    params.message = message;
 
-    return object->user_callback(
-        flags, object_type, object_handle, location, code, layer_prefix, message, object->user_data);
+    params.object_handle = wine_vk_get_wrapper(object->instance, object_handle);
+    if (!params.object_handle)
+        params.object_type = VK_DEBUG_REPORT_OBJECT_TYPE_UNKNOWN_EXT;
+
+    return KeUserModeCallback( NtUserCallVulkanDebugReportCallback, &params, sizeof(params),
+                               &ret_ptr, &ret_len );
 }
 
 static void wine_vk_physical_device_free(struct VkPhysicalDevice_T *phys_dev)
@@ -422,13 +434,10 @@ static void wine_vk_device_free(struct VkDevice_T *device)
     free(device);
 }
 
-NTSTATUS CDECL __wine_init_unix_lib(HMODULE module, DWORD reason, const void *driver, void *ptr_out)
+NTSTATUS init_vulkan(void *args)
 {
-    if (reason != DLL_PROCESS_ATTACH) return STATUS_SUCCESS;
-
-    vk_funcs = driver;
-    p_vkEnumerateInstanceVersion = vk_funcs->p_vkGetInstanceProcAddr(NULL, "vkEnumerateInstanceVersion");
-    *(const struct unix_funcs **)ptr_out = &loader_funcs;
+    vk_funcs = *(const struct vulkan_funcs **)args;
+    *(const struct unix_funcs **)args = &loader_funcs;
     return STATUS_SUCCESS;
 }
 
@@ -618,9 +627,12 @@ static void wine_vk_instance_free(struct VkInstance_T *instance)
     free(instance);
 }
 
-VkResult WINAPI wine_vkAllocateCommandBuffers(VkDevice device,
-        const VkCommandBufferAllocateInfo *allocate_info, VkCommandBuffer *buffers)
+NTSTATUS wine_vkAllocateCommandBuffers(void *args)
 {
+    struct vkAllocateCommandBuffers_params *params = args;
+    VkDevice device = params->device;
+    const VkCommandBufferAllocateInfo *allocate_info = params->pAllocateInfo;
+    VkCommandBuffer *buffers = params->pCommandBuffers;
     struct wine_cmd_pool *pool;
     VkResult res = VK_SUCCESS;
     unsigned int i;
@@ -674,10 +686,13 @@ VkResult WINAPI wine_vkAllocateCommandBuffers(VkDevice device,
     return res;
 }
 
-VkResult WINAPI wine_vkCreateDevice(VkPhysicalDevice phys_dev,
-        const VkDeviceCreateInfo *create_info,
-        const VkAllocationCallbacks *allocator, VkDevice *device)
+NTSTATUS wine_vkCreateDevice(void *args)
 {
+    struct vkCreateDevice_params *params = args;
+    VkPhysicalDevice phys_dev = params->physicalDevice;
+    const VkDeviceCreateInfo *create_info = params->pCreateInfo;
+    const VkAllocationCallbacks *allocator = params->pAllocator;
+    VkDevice *device = params->pDevice;
     VkDeviceCreateInfo create_info_host;
     struct VkQueue_T *next_queue;
     struct VkDevice_T *object;
@@ -769,9 +784,12 @@ fail:
     return res;
 }
 
-VkResult WINAPI wine_vkCreateInstance(const VkInstanceCreateInfo *create_info,
-        const VkAllocationCallbacks *allocator, VkInstance *instance)
+NTSTATUS wine_vkCreateInstance(void *args)
 {
+    struct vkCreateInstance_params *params = args;
+    const VkInstanceCreateInfo *create_info = params->pCreateInfo;
+    const VkAllocationCallbacks *allocator = params->pAllocator;
+    VkInstance *instance = params->pInstance;
     VkInstanceCreateInfo create_info_host;
     const VkApplicationInfo *app_info;
     struct VkInstance_T *object;
@@ -848,29 +866,44 @@ VkResult WINAPI wine_vkCreateInstance(const VkInstanceCreateInfo *create_info,
     return VK_SUCCESS;
 }
 
-void WINAPI wine_vkDestroyDevice(VkDevice device, const VkAllocationCallbacks *allocator)
+NTSTATUS wine_vkDestroyDevice(void *args)
 {
+    struct vkDestroyDevice_params *params = args;
+    VkDevice device = params->device;
+    const VkAllocationCallbacks *allocator = params->pAllocator;
+
     TRACE("%p %p\n", device, allocator);
 
     if (allocator)
         FIXME("Support for allocation callbacks not implemented yet\n");
 
     wine_vk_device_free(device);
+    return STATUS_SUCCESS;
 }
 
-void WINAPI wine_vkDestroyInstance(VkInstance instance, const VkAllocationCallbacks *allocator)
+NTSTATUS wine_vkDestroyInstance(void *args)
 {
+    struct vkDestroyInstance_params *params = args;
+    VkInstance instance = params->instance;
+    const VkAllocationCallbacks *allocator = params->pAllocator;
+
     TRACE("%p, %p\n", instance, allocator);
 
     if (allocator)
         FIXME("Support allocation allocators\n");
 
     wine_vk_instance_free(instance);
+    return STATUS_SUCCESS;
 }
 
-VkResult WINAPI wine_vkEnumerateDeviceExtensionProperties(VkPhysicalDevice phys_dev,
-        const char *layer_name, uint32_t *count, VkExtensionProperties *properties)
+NTSTATUS wine_vkEnumerateDeviceExtensionProperties(void *args)
 {
+    struct vkEnumerateDeviceExtensionProperties_params *params = args;
+    VkPhysicalDevice phys_dev = params->physicalDevice;
+    const char *layer_name = params->pLayerName;
+    uint32_t *count = params->pPropertyCount;
+    VkExtensionProperties *properties = params->pProperties;
+
     TRACE("%p, %p, %p, %p\n", phys_dev, layer_name, count, properties);
 
     /* This shouldn't get called with layer_name set, the ICD loader prevents it. */
@@ -893,9 +926,11 @@ VkResult WINAPI wine_vkEnumerateDeviceExtensionProperties(VkPhysicalDevice phys_
     return *count < phys_dev->extension_count ? VK_INCOMPLETE : VK_SUCCESS;
 }
 
-VkResult WINAPI wine_vkEnumerateInstanceExtensionProperties(const char *layer_name,
-        uint32_t *count, VkExtensionProperties *properties)
+NTSTATUS wine_vkEnumerateInstanceExtensionProperties(void *args)
 {
+    struct vkEnumerateInstanceExtensionProperties_params *params = args;
+    uint32_t *count = params->pPropertyCount;
+    VkExtensionProperties *properties = params->pProperties;
     uint32_t num_properties = 0, num_host_properties;
     VkExtensionProperties *host_properties;
     unsigned int i, j;
@@ -950,18 +985,27 @@ VkResult WINAPI wine_vkEnumerateInstanceExtensionProperties(const char *layer_na
     return *count < num_properties ? VK_INCOMPLETE : VK_SUCCESS;
 }
 
-VkResult WINAPI wine_vkEnumerateDeviceLayerProperties(VkPhysicalDevice phys_dev, uint32_t *count, VkLayerProperties *properties)
+NTSTATUS wine_vkEnumerateDeviceLayerProperties(void *args)
 {
-    TRACE("%p, %p, %p\n", phys_dev, count, properties);
+    struct vkEnumerateDeviceLayerProperties_params *params = args;
+    uint32_t *count = params->pPropertyCount;
+
+    TRACE("%p, %p, %p\n", params->physicalDevice, count, params->pProperties);
 
     *count = 0;
     return VK_SUCCESS;
 }
 
-VkResult WINAPI wine_vkEnumerateInstanceVersion(uint32_t *version)
+NTSTATUS wine_vkEnumerateInstanceVersion(void *args)
 {
+    struct vkEnumerateInstanceVersion_params *params = args;
+    uint32_t *version = params->pApiVersion;
     VkResult res;
 
+    static VkResult (*p_vkEnumerateInstanceVersion)(uint32_t *version);
+    if (!p_vkEnumerateInstanceVersion)
+        p_vkEnumerateInstanceVersion = vk_funcs->p_vkGetInstanceProcAddr(NULL, "vkEnumerateInstanceVersion");
+
     if (p_vkEnumerateInstanceVersion)
     {
         res = p_vkEnumerateInstanceVersion(version);
@@ -978,9 +1022,12 @@ VkResult WINAPI wine_vkEnumerateInstanceVersion(uint32_t *version)
     return res;
 }
 
-VkResult WINAPI wine_vkEnumeratePhysicalDevices(VkInstance instance, uint32_t *count,
-        VkPhysicalDevice *devices)
+NTSTATUS wine_vkEnumeratePhysicalDevices(void *args)
 {
+    struct vkEnumeratePhysicalDevices_params *params = args;
+    VkInstance instance = params->instance;
+    uint32_t *count = params->pPhysicalDeviceCount;
+    VkPhysicalDevice *devices = params->pPhysicalDevices;
     unsigned int i;
 
     TRACE("%p %p %p\n", instance, count, devices);
@@ -1001,14 +1048,18 @@ VkResult WINAPI wine_vkEnumeratePhysicalDevices(VkInstance instance, uint32_t *c
     return *count < instance->phys_dev_count ? VK_INCOMPLETE : VK_SUCCESS;
 }
 
-void WINAPI wine_vkFreeCommandBuffers(VkDevice device, VkCommandPool pool_handle,
-        uint32_t count, const VkCommandBuffer *buffers)
+NTSTATUS wine_vkFreeCommandBuffers(void *args)
 {
-    struct wine_cmd_pool *pool = wine_cmd_pool_from_handle(pool_handle);
+    struct vkFreeCommandBuffers_params *params = args;
+    VkDevice device = params->device;
+    struct wine_cmd_pool *pool = wine_cmd_pool_from_handle(params->commandPool);
+    uint32_t count = params->commandBufferCount;
+    const VkCommandBuffer *buffers = params->pCommandBuffers;
 
-    TRACE("%p, 0x%s, %u, %p\n", device, wine_dbgstr_longlong(pool_handle), count, buffers);
+    TRACE("%p, 0x%s, %u, %p\n", device, wine_dbgstr_longlong(params->commandPool), count, buffers);
 
     wine_vk_free_command_buffers(device, pool, count, buffers);
+    return STATUS_SUCCESS;
 }
 
 static VkQueue wine_vk_device_find_queue(VkDevice device, const VkDeviceQueueInfo2 *info)
@@ -1030,10 +1081,15 @@ static VkQueue wine_vk_device_find_queue(VkDevice device, const VkDeviceQueueInf
     return VK_NULL_HANDLE;
 }
 
-void WINAPI wine_vkGetDeviceQueue(VkDevice device, uint32_t family_index,
-        uint32_t queue_index, VkQueue *queue)
+NTSTATUS wine_vkGetDeviceQueue(void *args)
 {
+    struct vkGetDeviceQueue_params *params = args;
+    VkDevice device = params->device;
+    uint32_t family_index = params->queueFamilyIndex;
+    uint32_t queue_index = params->queueIndex;
+    VkQueue *queue = params->pQueue;
     VkDeviceQueueInfo2 queue_info;
+
     TRACE("%p, %u, %u, %p\n", device, family_index, queue_index, queue);
 
     queue_info.sType = VK_STRUCTURE_TYPE_DEVICE_QUEUE_INFO_2;
@@ -1043,10 +1099,15 @@ void WINAPI wine_vkGetDeviceQueue(VkDevice device, uint32_t family_index,
     queue_info.queueIndex = queue_index;
 
     *queue = wine_vk_device_find_queue(device, &queue_info);
+    return STATUS_SUCCESS;
 }
 
-void WINAPI wine_vkGetDeviceQueue2(VkDevice device, const VkDeviceQueueInfo2 *info, VkQueue *queue)
+NTSTATUS wine_vkGetDeviceQueue2(void *args)
 {
+    struct vkGetDeviceQueue2_params *params = args;
+    VkDevice device = params->device;
+    const VkDeviceQueueInfo2 *info = params->pQueueInfo;
+    VkQueue *queue = params->pQueue;
     const VkBaseInStructure *chain;
 
     TRACE("%p, %p, %p\n", device, info, queue);
@@ -1055,11 +1116,16 @@ void WINAPI wine_vkGetDeviceQueue2(VkDevice device, const VkDeviceQueueInfo2 *in
         FIXME("Ignoring a linked structure of type %u.\n", chain->sType);
 
     *queue = wine_vk_device_find_queue(device, info);
+    return STATUS_SUCCESS;
 }
 
-VkResult WINAPI wine_vkCreateCommandPool(VkDevice device, const VkCommandPoolCreateInfo *info,
-        const VkAllocationCallbacks *allocator, VkCommandPool *command_pool)
+NTSTATUS wine_vkCreateCommandPool(void *args)
 {
+    struct vkCreateCommandPool_params *params = args;
+    VkDevice device = params->device;
+    const VkCommandPoolCreateInfo *info = params->pCreateInfo;
+    const VkAllocationCallbacks *allocator = params->pAllocator;
+    VkCommandPool *command_pool = params->pCommandPool;
     struct wine_cmd_pool *object;
     VkResult res;
 
@@ -1088,16 +1154,19 @@ VkResult WINAPI wine_vkCreateCommandPool(VkDevice device, const VkCommandPoolCre
     return res;
 }
 
-void WINAPI wine_vkDestroyCommandPool(VkDevice device, VkCommandPool handle,
-        const VkAllocationCallbacks *allocator)
+NTSTATUS wine_vkDestroyCommandPool(void *args)
 {
+    struct vkDestroyCommandPool_params *params = args;
+    VkDevice device = params->device;
+    VkCommandPool handle = params->commandPool;
+    const VkAllocationCallbacks *allocator = params->pAllocator;
     struct wine_cmd_pool *pool = wine_cmd_pool_from_handle(handle);
     struct VkCommandBuffer_T *buffer, *cursor;
 
     TRACE("%p, 0x%s, %p\n", device, wine_dbgstr_longlong(handle), allocator);
 
     if (!handle)
-        return;
+        return STATUS_SUCCESS;
 
     if (allocator)
         FIXME("Support for allocation callbacks not implemented yet\n");
@@ -1116,6 +1185,7 @@ void WINAPI wine_vkDestroyCommandPool(VkDevice device, VkCommandPool handle,
 
     device->funcs.p_vkDestroyCommandPool(device->device, pool->command_pool, NULL);
     free(pool);
+    return STATUS_SUCCESS;
 }
 
 static VkResult wine_vk_enumerate_physical_device_groups(struct VkInstance_T *instance,
@@ -1143,57 +1213,88 @@ static VkResult wine_vk_enumerate_physical_device_groups(struct VkInstance_T *in
     return res;
 }
 
-VkResult WINAPI wine_vkEnumeratePhysicalDeviceGroups(VkInstance instance,
-        uint32_t *count, VkPhysicalDeviceGroupProperties *properties)
+NTSTATUS wine_vkEnumeratePhysicalDeviceGroups(void *args)
 {
+    struct vkEnumeratePhysicalDeviceGroups_params *params = args;
+    VkInstance instance = params->instance;
+    uint32_t *count = params->pPhysicalDeviceGroupCount;
+    VkPhysicalDeviceGroupProperties *properties = params->pPhysicalDeviceGroupProperties;
+
     TRACE("%p, %p, %p\n", instance, count, properties);
     return wine_vk_enumerate_physical_device_groups(instance,
             instance->funcs.p_vkEnumeratePhysicalDeviceGroups, count, properties);
 }
 
-VkResult WINAPI wine_vkEnumeratePhysicalDeviceGroupsKHR(VkInstance instance,
-        uint32_t *count, VkPhysicalDeviceGroupProperties *properties)
+NTSTATUS wine_vkEnumeratePhysicalDeviceGroupsKHR(void *args)
 {
+    struct vkEnumeratePhysicalDeviceGroupsKHR_params *params = args;
+    VkInstance instance = params->instance;
+    uint32_t *count = params->pPhysicalDeviceGroupCount;
+    VkPhysicalDeviceGroupProperties *properties = params->pPhysicalDeviceGroupProperties;
+
     TRACE("%p, %p, %p\n", instance, count, properties);
     return wine_vk_enumerate_physical_device_groups(instance,
             instance->funcs.p_vkEnumeratePhysicalDeviceGroupsKHR, count, properties);
 }
 
-void WINAPI wine_vkGetPhysicalDeviceExternalFenceProperties(VkPhysicalDevice phys_dev,
-        const VkPhysicalDeviceExternalFenceInfo *fence_info, VkExternalFenceProperties *properties)
+NTSTATUS wine_vkGetPhysicalDeviceExternalFenceProperties(void *args)
 {
+    struct vkGetPhysicalDeviceExternalFenceProperties_params *params = args;
+    VkPhysicalDevice phys_dev = params->physicalDevice;
+    const VkPhysicalDeviceExternalFenceInfo *fence_info = params->pExternalFenceInfo;
+    VkExternalFenceProperties *properties = params->pExternalFenceProperties;
+
     TRACE("%p, %p, %p\n", phys_dev, fence_info, properties);
     properties->exportFromImportedHandleTypes = 0;
     properties->compatibleHandleTypes = 0;
     properties->externalFenceFeatures = 0;
+    return STATUS_SUCCESS;
 }
 
-void WINAPI wine_vkGetPhysicalDeviceExternalFencePropertiesKHR(VkPhysicalDevice phys_dev,
-        const VkPhysicalDeviceExternalFenceInfo *fence_info, VkExternalFenceProperties *properties)
+NTSTATUS wine_vkGetPhysicalDeviceExternalFencePropertiesKHR(void *args)
 {
+    struct vkGetPhysicalDeviceExternalFencePropertiesKHR_params *params = args;
+    VkPhysicalDevice phys_dev = params->physicalDevice;
+    const VkPhysicalDeviceExternalFenceInfo *fence_info = params->pExternalFenceInfo;
+    VkExternalFenceProperties *properties = params->pExternalFenceProperties;
+
     TRACE("%p, %p, %p\n", phys_dev, fence_info, properties);
     properties->exportFromImportedHandleTypes = 0;
     properties->compatibleHandleTypes = 0;
     properties->externalFenceFeatures = 0;
+    return STATUS_SUCCESS;
 }
 
-void WINAPI wine_vkGetPhysicalDeviceExternalBufferProperties(VkPhysicalDevice phys_dev,
-        const VkPhysicalDeviceExternalBufferInfo *buffer_info, VkExternalBufferProperties *properties)
+NTSTATUS wine_vkGetPhysicalDeviceExternalBufferProperties(void *args)
 {
+    struct vkGetPhysicalDeviceExternalBufferProperties_params *params = args;
+    VkPhysicalDevice phys_dev = params->physicalDevice;
+    const VkPhysicalDeviceExternalBufferInfo *buffer_info = params->pExternalBufferInfo;
+    VkExternalBufferProperties *properties = params->pExternalBufferProperties;
+
     TRACE("%p, %p, %p\n", phys_dev, buffer_info, properties);
     memset(&properties->externalMemoryProperties, 0, sizeof(properties->externalMemoryProperties));
+    return STATUS_SUCCESS;
 }
 
-void WINAPI wine_vkGetPhysicalDeviceExternalBufferPropertiesKHR(VkPhysicalDevice phys_dev,
-        const VkPhysicalDeviceExternalBufferInfo *buffer_info, VkExternalBufferProperties *properties)
+NTSTATUS wine_vkGetPhysicalDeviceExternalBufferPropertiesKHR(void *args)
 {
+    struct vkGetPhysicalDeviceExternalBufferPropertiesKHR_params *params = args;
+    VkPhysicalDevice phys_dev = params->physicalDevice;
+    const VkPhysicalDeviceExternalBufferInfo *buffer_info = params->pExternalBufferInfo;
+    VkExternalBufferProperties *properties = params->pExternalBufferProperties;
+
     TRACE("%p, %p, %p\n", phys_dev, buffer_info, properties);
     memset(&properties->externalMemoryProperties, 0, sizeof(properties->externalMemoryProperties));
+    return STATUS_SUCCESS;
 }
 
-VkResult WINAPI wine_vkGetPhysicalDeviceImageFormatProperties2(VkPhysicalDevice phys_dev,
-        const VkPhysicalDeviceImageFormatInfo2 *format_info, VkImageFormatProperties2 *properties)
+NTSTATUS wine_vkGetPhysicalDeviceImageFormatProperties2(void *args)
 {
+    struct vkGetPhysicalDeviceImageFormatProperties2_params *params = args;
+    VkPhysicalDevice phys_dev = params->physicalDevice;
+    const VkPhysicalDeviceImageFormatInfo2 *format_info = params->pImageFormatInfo;
+    VkImageFormatProperties2 *properties = params->pImageFormatProperties;
     VkExternalImageFormatProperties *external_image_properties;
     VkResult res;
 
@@ -1212,9 +1313,12 @@ VkResult WINAPI wine_vkGetPhysicalDeviceImageFormatProperties2(VkPhysicalDevice
     return res;
 }
 
-VkResult WINAPI wine_vkGetPhysicalDeviceImageFormatProperties2KHR(VkPhysicalDevice phys_dev,
-        const VkPhysicalDeviceImageFormatInfo2 *format_info, VkImageFormatProperties2 *properties)
+NTSTATUS wine_vkGetPhysicalDeviceImageFormatProperties2KHR(void *args)
 {
+    struct vkGetPhysicalDeviceImageFormatProperties2KHR_params *params = args;
+    VkPhysicalDevice phys_dev = params->physicalDevice;
+    const VkPhysicalDeviceImageFormatInfo2 *format_info = params->pImageFormatInfo;
+    VkImageFormatProperties2 *properties = params->pImageFormatProperties;
     VkExternalImageFormatProperties *external_image_properties;
     VkResult res;
 
@@ -1279,10 +1383,14 @@ static inline uint64_t convert_timestamp(VkTimeDomainEXT host_domain, VkTimeDoma
     return value;
 }
 
-VkResult WINAPI wine_vkGetCalibratedTimestampsEXT(VkDevice device,
-    uint32_t timestamp_count, const VkCalibratedTimestampInfoEXT *timestamp_infos,
-    uint64_t *timestamps, uint64_t *max_deviation)
+NTSTATUS wine_vkGetCalibratedTimestampsEXT(void *args)
 {
+    struct vkGetCalibratedTimestampsEXT_params *params = args;
+    VkDevice device = params->device;
+    uint32_t timestamp_count = params->timestampCount;
+    const VkCalibratedTimestampInfoEXT *timestamp_infos = params->pTimestampInfos;
+    uint64_t *timestamps = params->pTimestamps;
+    uint64_t *max_deviation = params->pMaxDeviation;
     VkCalibratedTimestampInfoEXT* host_timestamp_infos;
     unsigned int i;
     VkResult res;
@@ -1310,9 +1418,12 @@ VkResult WINAPI wine_vkGetCalibratedTimestampsEXT(VkDevice device,
     return res;
 }
 
-VkResult WINAPI wine_vkGetPhysicalDeviceCalibrateableTimeDomainsEXT(VkPhysicalDevice phys_dev,
-    uint32_t *time_domain_count, VkTimeDomainEXT *time_domains)
+NTSTATUS wine_vkGetPhysicalDeviceCalibrateableTimeDomainsEXT(void *args)
 {
+    struct vkGetPhysicalDeviceCalibrateableTimeDomainsEXT_params *params = args;
+    VkPhysicalDevice phys_dev = params->physicalDevice;
+    uint32_t *time_domain_count = params->pTimeDomainCount;
+    VkTimeDomainEXT *time_domains = params->pTimeDomains;
     BOOL supports_device = FALSE, supports_monotonic = FALSE, supports_monotonic_raw = FALSE;
     const VkTimeDomainEXT performance_counter_domain = get_performance_counter_time_domain();
     VkTimeDomainEXT *host_time_domains;
@@ -1382,27 +1493,41 @@ VkResult WINAPI wine_vkGetPhysicalDeviceCalibrateableTimeDomainsEXT(VkPhysicalDe
     return res;
 }
 
-void WINAPI wine_vkGetPhysicalDeviceExternalSemaphoreProperties(VkPhysicalDevice phys_dev,
-        const VkPhysicalDeviceExternalSemaphoreInfo *semaphore_info, VkExternalSemaphoreProperties *properties)
+NTSTATUS wine_vkGetPhysicalDeviceExternalSemaphoreProperties(void *args)
 {
+    struct vkGetPhysicalDeviceExternalSemaphoreProperties_params *params = args;
+    VkPhysicalDevice phys_dev = params->physicalDevice;
+    const VkPhysicalDeviceExternalSemaphoreInfo *semaphore_info = params->pExternalSemaphoreInfo;
+    VkExternalSemaphoreProperties *properties = params->pExternalSemaphoreProperties;
+
     TRACE("%p, %p, %p\n", phys_dev, semaphore_info, properties);
     properties->exportFromImportedHandleTypes = 0;
     properties->compatibleHandleTypes = 0;
     properties->externalSemaphoreFeatures = 0;
+    return STATUS_SUCCESS;
 }
 
-void WINAPI wine_vkGetPhysicalDeviceExternalSemaphorePropertiesKHR(VkPhysicalDevice phys_dev,
-        const VkPhysicalDeviceExternalSemaphoreInfo *semaphore_info, VkExternalSemaphoreProperties *properties)
+NTSTATUS wine_vkGetPhysicalDeviceExternalSemaphorePropertiesKHR(void *args)
 {
+    struct vkGetPhysicalDeviceExternalSemaphorePropertiesKHR_params *params = args;
+    VkPhysicalDevice phys_dev = params->physicalDevice;
+    const VkPhysicalDeviceExternalSemaphoreInfo *semaphore_info = params->pExternalSemaphoreInfo;
+    VkExternalSemaphoreProperties *properties = params->pExternalSemaphoreProperties;
+
     TRACE("%p, %p, %p\n", phys_dev, semaphore_info, properties);
     properties->exportFromImportedHandleTypes = 0;
     properties->compatibleHandleTypes = 0;
     properties->externalSemaphoreFeatures = 0;
+    return STATUS_SUCCESS;
 }
 
-VkResult WINAPI wine_vkCreateWin32SurfaceKHR(VkInstance instance,
-        const VkWin32SurfaceCreateInfoKHR *createInfo, const VkAllocationCallbacks *allocator, VkSurfaceKHR *surface)
+NTSTATUS wine_vkCreateWin32SurfaceKHR(void *args)
 {
+    struct vkCreateWin32SurfaceKHR_params *params = args;
+    VkInstance instance = params->instance;
+    const VkWin32SurfaceCreateInfoKHR *createInfo = params->pCreateInfo;
+    const VkAllocationCallbacks *allocator = params->pAllocator;
+    VkSurfaceKHR *surface = params->pSurface;
     struct wine_surface *object;
     VkResult res;
 
@@ -1433,19 +1558,24 @@ VkResult WINAPI wine_vkCreateWin32SurfaceKHR(VkInstance instance,
     return VK_SUCCESS;
 }
 
-void WINAPI wine_vkDestroySurfaceKHR(VkInstance instance, VkSurfaceKHR surface, const VkAllocationCallbacks *allocator)
+NTSTATUS wine_vkDestroySurfaceKHR(void *args)
 {
+    struct vkDestroySurfaceKHR_params *params = args;
+    VkInstance instance = params->instance;
+    VkSurfaceKHR surface = params->surface;
+    const VkAllocationCallbacks *allocator = params->pAllocator;
     struct wine_surface *object = wine_surface_from_handle(surface);
 
     TRACE("%p, 0x%s, %p\n", instance, wine_dbgstr_longlong(surface), allocator);
 
     if (!object)
-        return;
+        return STATUS_SUCCESS;
 
     instance->funcs.p_vkDestroySurfaceKHR(instance->instance, object->driver_surface, NULL);
 
     WINE_VK_REMOVE_HANDLE_MAPPING(instance, object);
     free(object);
+    return STATUS_SUCCESS;
 }
 
 static inline void adjust_max_image_count(VkPhysicalDevice phys_dev, VkSurfaceCapabilitiesKHR* capabilities)
@@ -1463,9 +1593,12 @@ static inline void adjust_max_image_count(VkPhysicalDevice phys_dev, VkSurfaceCa
     }
 }
 
-VkResult WINAPI wine_vkGetPhysicalDeviceSurfaceCapabilitiesKHR(VkPhysicalDevice phys_dev,
-        VkSurfaceKHR surface, VkSurfaceCapabilitiesKHR *capabilities)
+NTSTATUS wine_vkGetPhysicalDeviceSurfaceCapabilitiesKHR(void *args)
 {
+    struct vkGetPhysicalDeviceSurfaceCapabilitiesKHR_params *params = args;
+    VkPhysicalDevice phys_dev = params->physicalDevice;
+    VkSurfaceKHR surface = params->surface;
+    VkSurfaceCapabilitiesKHR *capabilities = params->pSurfaceCapabilities;
     VkResult res;
 
     TRACE("%p, 0x%s, %p\n", phys_dev, wine_dbgstr_longlong(surface), capabilities);
@@ -1478,9 +1611,12 @@ VkResult WINAPI wine_vkGetPhysicalDeviceSurfaceCapabilitiesKHR(VkPhysicalDevice
     return res;
 }
 
-VkResult WINAPI wine_vkGetPhysicalDeviceSurfaceCapabilities2KHR(VkPhysicalDevice phys_dev,
-        const VkPhysicalDeviceSurfaceInfo2KHR *surface_info, VkSurfaceCapabilities2KHR *capabilities)
+NTSTATUS wine_vkGetPhysicalDeviceSurfaceCapabilities2KHR(void *args)
 {
+    struct vkGetPhysicalDeviceSurfaceCapabilities2KHR_params *params = args;
+    VkPhysicalDevice phys_dev = params->physicalDevice;
+    const VkPhysicalDeviceSurfaceInfo2KHR *surface_info = params->pSurfaceInfo;
+    VkSurfaceCapabilities2KHR *capabilities = params->pSurfaceCapabilities;
     VkResult res;
 
     TRACE("%p, %p, %p\n", phys_dev, surface_info, capabilities);
@@ -1493,9 +1629,13 @@ VkResult WINAPI wine_vkGetPhysicalDeviceSurfaceCapabilities2KHR(VkPhysicalDevice
     return res;
 }
 
-VkResult WINAPI wine_vkCreateDebugUtilsMessengerEXT(VkInstance instance, const VkDebugUtilsMessengerCreateInfoEXT *create_info,
-        const VkAllocationCallbacks *allocator, VkDebugUtilsMessengerEXT *messenger)
+NTSTATUS wine_vkCreateDebugUtilsMessengerEXT(void *args)
 {
+    struct vkCreateDebugUtilsMessengerEXT_params *params = args;
+    VkInstance instance = params->instance;
+    const VkDebugUtilsMessengerCreateInfoEXT *create_info = params->pCreateInfo;
+    const VkAllocationCallbacks *allocator = params->pAllocator;
+    VkDebugUtilsMessengerEXT *messenger = params->pMessenger;
     VkDebugUtilsMessengerCreateInfoEXT wine_create_info;
     struct wine_debug_utils_messenger *object;
     VkResult res;
@@ -1531,9 +1671,12 @@ VkResult WINAPI wine_vkCreateDebugUtilsMessengerEXT(VkInstance instance, const V
     return VK_SUCCESS;
 }
 
-void WINAPI wine_vkDestroyDebugUtilsMessengerEXT(
-        VkInstance instance, VkDebugUtilsMessengerEXT messenger, const VkAllocationCallbacks *allocator)
+NTSTATUS wine_vkDestroyDebugUtilsMessengerEXT(void *args)
 {
+    struct vkDestroyDebugUtilsMessengerEXT_params *params = args;
+    VkInstance instance = params->instance;
+    VkDebugUtilsMessengerEXT messenger = params->messenger;
+    const VkAllocationCallbacks *allocator = params->pAllocator;
     struct wine_debug_utils_messenger *object;
 
     TRACE("%p, 0x%s, %p\n", instance, wine_dbgstr_longlong(messenger), allocator);
@@ -1541,17 +1684,22 @@ void WINAPI wine_vkDestroyDebugUtilsMessengerEXT(
     object = wine_debug_utils_messenger_from_handle(messenger);
 
     if (!object)
-        return;
+        return STATUS_SUCCESS;
 
     instance->funcs.p_vkDestroyDebugUtilsMessengerEXT(instance->instance, object->debug_messenger, NULL);
     WINE_VK_REMOVE_HANDLE_MAPPING(instance, object);
 
     free(object);
+    return STATUS_SUCCESS;
 }
 
-VkResult WINAPI wine_vkCreateDebugReportCallbackEXT(VkInstance instance, const VkDebugReportCallbackCreateInfoEXT *create_info,
-    const VkAllocationCallbacks *allocator, VkDebugReportCallbackEXT *callback)
+NTSTATUS wine_vkCreateDebugReportCallbackEXT(void *args)
 {
+    struct vkCreateDebugReportCallbackEXT_params *params = args;
+    VkInstance instance = params->instance;
+    const VkDebugReportCallbackCreateInfoEXT *create_info = params->pCreateInfo;
+    const VkAllocationCallbacks *allocator = params->pAllocator;
+    VkDebugReportCallbackEXT *callback = params->pCallback;
     VkDebugReportCallbackCreateInfoEXT wine_create_info;
     struct wine_debug_report_callback *object;
     VkResult res;
@@ -1587,9 +1735,12 @@ VkResult WINAPI wine_vkCreateDebugReportCallbackEXT(VkInstance instance, const V
     return VK_SUCCESS;
 }
 
-void WINAPI wine_vkDestroyDebugReportCallbackEXT(
-    VkInstance instance, VkDebugReportCallbackEXT callback, const VkAllocationCallbacks *allocator)
+NTSTATUS wine_vkDestroyDebugReportCallbackEXT(void *args)
 {
+    struct vkDestroyDebugReportCallbackEXT_params *params = args;
+    VkInstance instance = params->instance;
+    VkDebugReportCallbackEXT callback = params->callback;
+    const VkAllocationCallbacks *allocator = params->pAllocator;
     struct wine_debug_report_callback *object;
 
     TRACE("%p, 0x%s, %p\n", instance, wine_dbgstr_longlong(callback), allocator);
@@ -1597,13 +1748,14 @@ void WINAPI wine_vkDestroyDebugReportCallbackEXT(
     object = wine_debug_report_callback_from_handle(callback);
 
     if (!object)
-        return;
+        return STATUS_SUCCESS;
 
     instance->funcs.p_vkDestroyDebugReportCallbackEXT(instance->instance, object->debug_callback, NULL);
 
     WINE_VK_REMOVE_HANDLE_MAPPING(instance, object);
 
     free(object);
+    return STATUS_SUCCESS;
 }
 
 BOOL WINAPI wine_vk_is_available_instance_function(VkInstance instance, const char *name)
